/**
 * Creates an instance of GeoJSON.
 *
 * @author Vignesh Gopalakrishnan
 * @constructor
 * @param {map, props} Map Instance and Addt prop object..
 * @this {GeoJSON}
 */
/* global LocDS */
/* global  Scripting */
/* global  L */
/* global Choropleth */

function GeoJSON(map, props) {
  var mapContainer = map;
  var addProps = props;
  var util = new VbiUtil();
  this.getMapContainer = function() {
    return mapContainer;
  };

  this.getProps = function() {
    return addProps;
  };

  this.getUtil = function() {
    return util;
  };
}


GeoJSON.prototype.deleteLayer = function(layer){
  var self = this;
  var component = self.getProps();
  if(layer){
    var id = L.Util.stamp(layer);
    var obj = self.getMapContainer()._layers[id];
    if (obj && self.getMapContainer().hasLayer(obj)) {
      self.getMapContainer().removeLayer(obj);
      component.layerControl.removeLayer(layer);
      delete self.getMapContainer()._layers[id];
    }
  }
};

/**
 * Will check where GeoJSON is gonna fetched and will be loaded to map.
 *
 * @return {null} .
 */
GeoJSON.prototype.loadGeoJSON = function() {
  var self = this;
  var component = self.getProps();
  //var dat;
  var geoJSONLayers;
  var conditionalFormatting;
  if (component.cfButton()) {
    conditionalFormatting = JSON.parse(component.cfButton());
  }

  if (component.geoJSONLayerContainer()) {
    geoJSONLayers = JSON.parse(component.geoJSONLayerContainer());
  }
  if(component.backUpChoropleth && component.choroplethLayers() && Object.keys(JSON.parse(component.choroplethLayers())).length == 0 ){
    for(var name in component.backUpChoropleth){
      if(component.backUpChoropleth[name]){
        self.deleteLayer(component.markerReference[component.backUpChoropleth[name].layer+'|'+component.backUpChoropleth[name].name]);
        delete component.markerReference[component.backUpChoropleth[name].layer+'|'+component.backUpChoropleth[name].name];
        $('#'+component.backUpChoropleth[name].name+'-cho').remove();
      }
    }
  }

  if (component.useChoropleth() && component.choroplethLayers() && Object.keys(JSON.parse(component.choroplethLayers())).length > 0) {
    var choroplethLayer = JSON.parse(component.choroplethLayers());
    var conditionalFormatting;
    if (component.cfButton()) {
      conditionalFormatting = JSON.parse(component.cfButton());
    }
    //Iterating the conditional formatting rule and adding it to the layer object.
    for (var rule in conditionalFormatting) {
      //var layerName = conditionalFormatting[rule].mapLayer.split('-')[0];
      var layerType = conditionalFormatting[rule].mapLayer.split('-')[1];
      for (var lyr in choroplethLayer) {
        if (choroplethLayer[lyr].cf === undefined){
        choroplethLayer[lyr].cf = [];}

        if (layerType == 'choropleth') {
          choroplethLayer[lyr].cf.push(conditionalFormatting[rule]);
        }
      }
    }

    for (var name in choroplethLayer) {
      if(choroplethLayer[name].layer===undefined){
        choroplethLayer[name].layer = 'chloropleth';
      }

      if(component.backUpChoropleth && component.backUpChoropleth[name]){
        delete component.backUpChoropleth[name].geo;
      }

      if(component.backUpChoropleth && JSON.stringify(component.backUpChoropleth[name])==JSON.stringify(choroplethLayer[name])){
        //delete choroplethLayer[name];
      }else if(component.backUpChoropleth && JSON.stringify(component.backUpChoropleth[name])!=JSON.stringify(choroplethLayer[name])){
        self.deleteLayer(component.markerReference[choroplethLayer[name].layer+'|'+choroplethLayer[name].name]);
        delete component.markerReference[choroplethLayer[name].layer+'|'+choroplethLayer[name].name];
        $('#'+choroplethLayer[name].name+'-cho').remove();
      }
    }

    if(component.backUpChoropleth && JSON.stringify(component.backUpChoropleth)!=JSON.stringify(choroplethLayer)){
        for(var name in component.backUpChoropleth){
          if(choroplethLayer[name]===undefined){
            self.deleteLayer(component.markerReference[component.backUpChoropleth[name].layer+'|'+component.backUpChoropleth[name].name]);
            delete component.markerReference[component.backUpChoropleth[name].layer+'|'+component.backUpChoropleth[name].name];
            $('#'+component.backUpChoropleth[name].name+'-cho').remove();
          }
        }
    }

    for (var name in choroplethLayer){
      if(component.backUpChoropleth && JSON.stringify(component.backUpChoropleth[name])==JSON.stringify(choroplethLayer[name])){
        delete choroplethLayer[name];
      }
    }

    if(component.backUpChoropleth==null){
      component.backUpChoropleth = choroplethLayer;
    }

    for (var name in choroplethLayer){
      if(choroplethLayer[name]){
        for (var layer in geoJSONLayers) {
          if (choroplethLayer[name].geoLayer == geoJSONLayers[layer].name) {
            choroplethLayer[name].geo = geoJSONLayers[layer];
            self.getAndLoadGeoJson(choroplethLayer[name]);
            $('.leaflet-control-layers').css('visibility', 'visible');
          }
        }
      }
    }

  }else {
    //Comparing and loading only once.
    for (var layer in geoJSONLayers) {
      geoJSONLayers[layer].layer = 'geo';

      if(component.backUpGeoJSON && JSON.stringify(component.backUpGeoJSON[layer])==JSON.stringify(geoJSONLayers[layer])){
        delete geoJSONLayers[layer];
      }else if(component.backUpGeoJSON && JSON.stringify(component.backUpGeoJSON[layer])!=JSON.stringify(geoJSONLayers[layer])){
        self.deleteLayer(component.markerReference[geoJSONLayers[layer].layer+'|'+geoJSONLayers[layer].name]);
        delete component.markerReference[geoJSONLayers[layer].layer+'|'+geoJSONLayers[layer].name];
      }

      if(geoJSONLayers[layer]){
        self.getAndLoadGeoJson(geoJSONLayers[layer]);
      }
      
      $('.leaflet-control-layers').css('visibility', 'visible');
    }
    if(component.backUpGeoJSON==null){
      component.backUpGeoJSON = geoJSONLayers;
    }
  }

  //Based on User's action it will hide layer control icon.
  if (component.isHideLayerControl()) {
    $('.leaflet-control-layers').css('visibility', 'hidden');
  } else {
    $('.leaflet-control-layers').css('visibility', 'visible');
  }
};

/**
 * Given geojson layer,  it will fetch the geojson object and load in the map.
 * All the switch cases are self explanatory.
 * @return {null}.
 *
 */
GeoJSON.prototype.getAndLoadGeoJson = function(geoJSONLyr) {
  var choroLyr = geoJSONLyr;
  if (geoJSONLyr.geoLayer) {
    geoJSONLyr = geoJSONLyr.geo;

  }

  var self = this;
  //var component = self.getProps();
  switch (geoJSONLyr.option) {
    case 'url':
      $.ajax({
        url: geoJSONLyr.url,
        dataType: 'json',
        async: false,
        success: function(data) {
          self.loadGeoJSONObj(data, choroLyr);
        }
      });
      break;
    case 'rawText':
      self.loadGeoJSONObj(JSON.parse(geoJSONLyr.rawText), choroLyr);
      break;
    case 'file':
      var fileURL = geoJSONLyr.file;
      if (fileURL) {
        $.ajax({
          url: fileURL,
          dataType: 'json',
          async: false,
          success: function(data) {
            self.loadGeoJSONObj(data, choroLyr);
          }
        });
      }

      break;
    case 'repo':
      var fileURL = geoJSONLyr.repoFile;
      if (fileURL) {
        $.ajax({
          dataType: 'json',
          url: fileURL,
          async: false,
          success: function(data) {
            self.loadGeoJSONObj(data, choroLyr);
          }
        });
      }
  }

};
/**
 * Given URL, GeoJSON will be fetched.
 *
 * @return {JSON} .
 */
GeoJSON.prototype.fetchJSONFromURL = function(url) {
  return $.getJSON(url);
};
/**
 * Given GeoJSON, it will be loaded in the map.
 *
 * @return {null} .
 */
GeoJSON.prototype.loadGeoJSONObj = function(obj, layerObj) {

  var self = this;
  var component = self.getProps();
  var datasource = LocDS.getInstance(), keyText, dataConfigs, currentResultSet;
  if(component.isDataProvider() && component.locDatasourceProperties()){
    dataConfigs= JSON.parse(component.locDatasourceProperties());
  }

  if(!component.isDataProvider()){
    component.dataProviderArray = [];
    component.dataProviderArray.push('VBX_DEFAULT_DS');
  }

  var scripting = new Scripting();
  var choropleth = new Choropleth(self.getMapContainer(), self.getProps());
  var choroLyr;
  if (layerObj.geoLayer) {
    choroLyr = layerObj;
    layerObj = choroLyr.geo;
  }

  //Supporting legacy.  If fDimension variable is not there create it manually
  if (choroLyr && !choroLyr.fDimension) {
    choroLyr.fDimension = 'No Filter';
  }

  if (layerObj.keyText === '') {
    keyText = 'text';
  } else {
    keyText = layerObj.keyText;
  }

  var dsRows = null;
  var measure = null;
  var dimension = null;

  //iterating all the multiple data's
for (var z = 0; z < component.dataProviderArray.length; z++) {

  var dataProviderID = component.dataProviderArray[z];

  if(choroLyr && choroLyr.datasource && choroLyr.datasource!=dataProviderID){
    continue;
  }
  
  if(dataProviderID == layerObj.datasource){ 
    currentResultSet = JSON.parse(component.getDataProviderResult(dataProviderID));
  }else{
    currentResultSet = null;
  }

  //If user enabled multiple data then loop will enter this if loop
  if(dataProviderID != 'VBX_DEFAULT_DS' && choroLyr && choroLyr.datasource==dataProviderID && layerObj.datasource == choroLyr.datasource){
   currentResultSet = JSON.parse(component.getDataProviderResult(dataProviderID));
   component.datasourceUsed.push(dataProviderID);
  }else if(dataProviderID == 'VBX_DEFAULT_DS'){
    //Checking if the resultset is there.  else breaking the whole operation.
    if(component.resultset()){
     currentResultSet = component.resultset();      
    }else{
      break;
    }
  }


  
  if(currentResultSet){
    dsRows = datasource.getDMKV(currentResultSet, keyText);
    measure = datasource.getMeasureName(currentResultSet, keyText);
    dimension = datasource.getDimensionObj(currentResultSet);

  try {
    //If color is not defined we are assigning transparent color to it.
    if (layerObj.fillColor == 'No Fill' || layerObj.fillColor == "rgba(255, 255, 255, 0)" || layerObj.fillColor == "") {
      layerObj.fillColor = 'transparent';
    }

    if (layerObj.color == 'No Fill' || layerObj.color == "rgba(255, 255, 255, 0)" || layerObj.color == "") {
      layerObj.color = 'transparent';
    }

    var geoJSON = L.geoJson(obj, {
      style: {
        color: layerObj.color,
        weight: parseFloat(layerObj.weight) || 5,
        opacity: parseFloat(layerObj.opacity) || 0.65,
        fillColor: layerObj.fillColor
      }
    }).addTo(self.getMapContainer());
    geoJSON.eachLayer(function(layer) {
      layer.setStyle({fillColor: layerObj.fillColor});
    });

    geoJSON.eachLayer(function(layer) {

      layer.row = null;
      layer.self = component;

      if (component.useChoropleth() && choroLyr) {
        layer.geoJSON = true;
        layer.dimensionObj = dimension;
        layer.dimension = layerObj.dimension;
        layer.dimValue = layer.feature.properties[layerObj.properties];
        layer.on({click: scripting.callDSScripting});
        choropleth.setFillColor(dsRows, measure, layer.feature.properties[layerObj.properties], layerObj, layer, choroLyr);
      }

      var content = '';
      if (component.browser && layer.feature && layer.feature.properties && layer.feature.properties[layerObj.properties]) {
        content = self.generateTooltipContent(dsRows, measure, layer.feature.properties[layerObj.properties], layerObj, layer, choroLyr);
        layer.row = layerObj;
        if (content) {
          layer.bindPopup(content);
        }
      }
    });

    geoJSON.on('click', function(e){
      if(choroLyr.selectedColor=="rgba(255, 255, 255, 0)" || choroLyr.selectedColor===""){
        choroLyr.selectedColor = 'transparent';
      }

      if(component.useChoropleth() && choroLyr && choroLyr.selectedColor && e.layer && e.layer.options){
        if(!e.layer.options.old && e.layer.options.fillColor){
          e.layer.setStyle({'fillColor':choroLyr.selectedColor,"old":e.layer.options.fillColor});
        }else if(e.layer.options.old){
          e.layer.setStyle({'fillColor':e.layer.options.old});
          delete e.layer.options.old;
        }
      }
    });

    geoJSON.on('mouseover', function(e) {
      e.layer.openPopup();
    });

    geoJSON.on('mouseout', function(e) {
      e.layer.closePopup();
    });
    //Adding legend to Map for choropleth color
    if (component.useChoropleth() && choroLyr) {
      choropleth.addLegend(measure,layerObj,choroLyr);
    }

     //GeoJSON/ Choropleth layer name
      var layerName = layerObj.name;
      var layerType = layerObj.layer;
      if (choroLyr) {
        layerName = choroLyr.name;
        layerType = choroLyr.layer;
      }

    if(component.markerReference[layerType+'|'+layerName]==undefined){
        component.layerControl.addOverlay(geoJSON, layerName);
        //Adding the object to global array which will be helpful to delte the object when we recreate the map.
        component.markerReference[layerType+'|'+layerName] = geoJSON;
        //Adding all Layers name to an array
        if ($.inArray(layerName, component.allLayers) == -1) {
          component.allLayers.push(layerName);
        }

        if (choroLyr && !choroLyr.isLoad) {
          self.getMapContainer().removeLayer(geoJSON);
          component.layerVisibility[layerName] = false;
        }else{
           component.layerVisibility[layerName] = true;
        }

        if (!layerObj.isLoad) {
          self.getMapContainer().removeLayer(geoJSON);
          component.layerVisibility[layerName] = false;
        }  //self.generateAttributeHTML(obj);
        
        component.leafletLayerStatus(JSON.stringify(component.layerVisibility));
        component.firePropertiesChanged(['leafletLayerStatus']);
    }
  } catch (e) {
    if (window.console) {
      console.log(e); // jshint ignore:line
    }
  }
}//If Loop
}//for loop multiple datasource
};
/**
 * Generates content for GEOJson Tooltip.
 *
 * @return {TooltipContnet} .
 */
GeoJSON.prototype.generateTooltipContent = function(dsRows, measure, tooltipValue, layerObj, layer, choroLyr) {
  //var self = this;
  //var component = self.getProps();
  var measureAggregation = {};
  var content = '';
  var isMatched = false;
  //Setting up default values as zero
  for (var m in measure) {
    measureAggregation[measure[m]] = 0;
  }

  layer.matchedRows = [];
  //Iterating the datasource to match the geojson value.
  for (var key in dsRows) {
    if (dsRows[key][layerObj.dimension] && tooltipValue && dsRows[key][layerObj.dimension] == tooltipValue) {
      isMatched = true;
      var isResult = false;
      //Finding whether it's a result row.
      for (var res in dsRows[key]) {
        if (dsRows[key][res] == 'Result' || dsRows[key][res] == 'Overall Result') {
          isResult = true;
          break;
        }
      }
      //If it's a result row omitting it.
      if (!isResult) {
        for (var m in measure) {
          measureAggregation[measure[m]] = parseFloat(measureAggregation[measure[m]]) + parseFloat(dsRows[key][m]);
        }

        layer.matchedRows.push(dsRows[key]);
      }
    }
  }

  layer.aggregatedMeasure = measureAggregation;
  //Iterating the aggregated measure and writing the html content
  for (var key in measureAggregation) {
    //This if condition added to solve VBX141-1890 
    if (choroLyr && measure[choroLyr.measure] == key && isMatched) {
      if ((choroLyr.fDimension == 'No Filter' || (choroLyr.fDimension == choroLyr.geo.dimension && $.inArray(layer.feature.properties[layerObj.properties], choroLyr.filter) > -1)) && choroLyr.geoLayer == layerObj.name) {
        if (content === '') {
          content = '<b>' + key + '</b> : ' + measureAggregation[key].toFixed(choroLyr.significand);
        } else {
          content = content + '</br> <b>' + key + '</b> : ' + measureAggregation[key].toFixed(choroLyr.significand);
        }
      }else{
        if (content === '') {
          content = '<b>' + key + '</b> : ' + measureAggregation[key];
        } else {
          content = content + '</br> <b>' + key + '</b> : ' + measureAggregation[key];
        }
      }
    }
  }

  if (tooltipValue) {
    return '<b>' + tooltipValue + '</b></br>' + content;
  } else {
    return '';
  }
};
/**
 * Generates HTML content on attributes for given geoJSON.
 *
 * @return {NULL} .
 */
GeoJSON.prototype.generateAttributeHTML = function(obj) {
  var self = this;
  var component = self.getProps();
  var isHeading = false;
  var heading = '';
  var props = [];
  var html = '';
  for (var feature in obj.features) {
    html = html + '<tr>';
    for (var prop in obj.features[feature].properties) {
      if (!isHeading) {
        heading = heading + '<th>' + prop + '</th>';
        props.push(prop);
      }

      html = html + '<td>' + obj.features[feature].properties[prop] + '</td>';
    }

    isHeading = true;
    html = html + '</tr>';
  }

  component.geoJSONAttribute(JSON.stringify(props) + '||<table class=\'table table-hover\' border=\'1\' style=\'border-collapse: collapse;\'>' + heading + html + '</table>');  //component.firePropertiesChanged(["geoJSONAttribute"]);
};
