/**
 * Creates an instance of Choropleth.
 *
 * @author Vignesh Gopalakrishnan
 * @constructor
 * @param {map, props} Map Instance and Addt prop object..
 * @this {Choropleth}
 */
/* global chroma */
/* global  d3 */
/* global  L */
function Choropleth(map, props) {
  var mapContainer = map;
  var addProps = props;
  this.getMapContainer = function() {
    return mapContainer;
  };

  this.getProps = function() {
    return addProps;
  };
}
/**
 * The function will just set the fill color on the geojson layer based on the choropleth.
 *
 * @return {null} .
 */
Choropleth.prototype.setFillColor = function(dsRows, measure, tooltipValue, layerObj, layer, choroLyr) {
  var self = this;
  var component = self.getProps();
  //var measureAggregation = {};
  //Get the choropleth objects from additional properties.
  //var choroplethLayer = JSON.parse(component.choroplethLayers());
  //If the geojson layer name and it's name mapped to choropleth matches then proceed.
  if ((choroLyr.fDimension == 'No Filter' || (choroLyr.fDimension == choroLyr.geo.dimension && $.inArray(layer.feature.properties[layerObj.properties], choroLyr.filter) > -1)) && choroLyr.geoLayer == layerObj.name) {
    //This maxMin contains the array of values for all measures.
    var maxMin = self.getMaxMin(dsRows, measure, layerObj, choroLyr);
    component.measureArray = maxMin;
    //The qlqz might have 4 different options quantile/quantize/linear/custom.  Where in first three are in one bucket and second two in other bucket.
    // Source: https://github.com/mbostock/d3/wiki/Quantitative-Scales
    var qlqz = choroLyr.qlOrQz;
    var quantileScales;
    //The if condition is just to mask the buckets.  I know there is some confusion.
    if (qlqz == 'quantile' || qlqz == 'quantize' || qlqz == 'linear') {
      quantileScales = 'quantile';
    } else if (qlqz == 'custom') {
      quantileScales = 'custom';
    }

    var choroplethSelectedMeasureValue;
    if (component.stateWiseAggregation[layerObj.name + choroLyr.name + 'StateAggregation'][tooltipValue]) {
      choroplethSelectedMeasureValue = component.stateWiseAggregation[layerObj.name + choroLyr.name + 'StateAggregation'][tooltipValue][measure[choroLyr.measure]];
    }

    //Manipulating based on quantile / quantize.
    switch (quantileScales) {
    case 'quantile':
      //The array will store the quantile color's selected by user in addt. props
      var quantileColor = [];
      //Pushing all the colors.
      for (var key in choroLyr.quantile) {
        quantileColor.push(choroLyr.quantile[key].color);
      }

      if (quantileColor.length == 1) {
        quantileColor.push(quantileColor[0]);
      }

      //If the state has the value then will be setting the fill color to the choropleth.
      if (component.stateWiseAggregation[layerObj.name + choroLyr.name + 'StateAggregation'][tooltipValue]) {
        //The following variable contains the selected measure value.
        // var choroplethSelectedMeasureValue = component.stateWiseAggregation[layerObj.name + 'StateAggregation'][tooltipValue][measure[choroLyr.measure]];
        var scale;
        //Just using chroma as color brewer.
        //This will run for each and every geojson object.  Need more optimization here to make it faster.

        var msrArray = self.unique(component.measureArray[measure[choroLyr.measure]]).sort(function(a, b) {
            return a - b;
          });

        var colorScale;
        if (choroLyr.nature != 'stan' && choroLyr.colorChoice == 'brewer') {
          colorScale = chroma.scale(choroLyr.brewerColor).colors(_.parseInt(choroLyr.buckets));
        }else if (choroLyr.nature == 'stan' && choroLyr.colorChoice == 'brewer') {
          colorScale = chroma.scale(component.colorBrewer[choroLyr.brewerColor]).colors(_.parseInt(choroLyr.buckets));
        }else {
          colorScale = chroma.scale(quantileColor).colors(msrArray.length);
        }

        //Calculating all these values from d3. Refer https://github.com/mbostock/d3/wiki/Quantitative-Scales for more details.
        if (qlqz == 'quantile') {
          //Calling d3 for Quantile Scales;
          scale = d3.scale.quantile().domain(msrArray).range(colorScale);
        } else if (qlqz == 'quantize') {
          //Calling d3 for Quantize Scales;
          scale = d3.scale.quantize().domain(msrArray).range(colorScale);
        } else if (qlqz == 'linear') {
          //Calling d3 for linear Scales;
          scale = d3.scale.linear().domain(msrArray).range(colorScale);
        }
        //Setting the fill color of the layer.
        layer.setStyle({fillColor: scale(choroplethSelectedMeasureValue)});
      }

      break;
    case 'custom':
      //The following value will have all the custom value chosed by user in addt. props.
      //Masking the quantize option chose by user into the custom here, just to make life easier.
      var quantizeColorArray = choroLyr.quantize;
      //Pushing all the colors.
      for (var key in quantizeColorArray) {
        if (component.stateWiseAggregation[layerObj.name + choroLyr.name + 'StateAggregation'][tooltipValue]) {
          //The following variable contains the selected measure value.
          //var choroplethSelectedMeasureValue = component.stateWiseAggregation[layerObj.name + 'StateAggregation'][tooltipValue][measure[choroLyr.measure]];
          //If the value lies in between this bracket, then layer's fill color will be changed.
          if (choroplethSelectedMeasureValue >= quantizeColorArray[key].start && choroplethSelectedMeasureValue <= quantizeColorArray[key].stop) {
            //Setting the fill color of the layer.
            layer.setStyle({
                fillColor: quantizeColorArray[key].color,
                opacity: parseFloat(quantizeColorArray[key].opacity)
              });
          }
        }
      }

      break;
    }

    //Below if is Completely for conditional formatting - If and only if the conditional formatting is undefined it will work.
    if (choroLyr.cf) {
      for (var cond in choroLyr.cf) {
        var layerName = choroLyr.cf[cond].mapLayer.split('-')[0];

        if (choroLyr.name == layerName) {
          var currentChoro = choroLyr.cf[cond];
          //This variable will change based on the comparison value type.
          var cValue,isCondition,cfColor;
          if (currentChoro.cValueType == 'static') {
            cValue = currentChoro.cValue;
          }else if (currentChoro.cValueType == 'dynamic') {
            if (currentChoro.cDynamicType == 'cell') {
              cValue = currentChoro.dValue[0];
            }else {
              if (undefined === component.stateWiseAggregation[layerObj.name + choroLyr.name + 'StateAggregation'][tooltipValue]){
              cValue = 0;}
              if (component.stateWiseAggregation[layerObj.name + choroLyr.name + 'StateAggregation'] && component.stateWiseAggregation[layerObj.name + choroLyr.name + 'StateAggregation'][tooltipValue]){
              cValue = component.stateWiseAggregation[layerObj.name + choroLyr.name + 'StateAggregation'][tooltipValue][measure[currentChoro.cDynamicMeasure]];}
              else{
			  cValue = 0;}
              //cValue = dsRows[text[count]][choroLyr.cf.cDynamicMeasure];
            }
          }

          //Based on the rule type the different conditions will be executed.
          switch (currentChoro.ruleType){
            case 'single':
              //The operator user chooses is used here.
              //I know using eval is not right.  But no other go for me here to evaluavate this.  That's okay.
              if (currentChoro.operator != 'btw') {
                isCondition = eval(parseFloat(choroplethSelectedMeasureValue) + currentChoro.operator + parseFloat(cValue)); // jshint ignore:line
                cfColor = currentChoro.color;
              }else {
                if (currentChoro.cValueType == 'static') {
                  if (parseFloat(choroplethSelectedMeasureValue) >= currentChoro.cfbtwStartStatic && parseFloat(choroplethSelectedMeasureValue) <= currentChoro.cfbtwEndStatic) {
                    isCondition = true;
                    cfColor = currentChoro.color;
                  }
                }else if (currentChoro.cValueType == 'dynamic') {
                  switch (currentChoro.cDynamicType){
                    case 'cell':
                      if (parseFloat(choroplethSelectedMeasureValue) >= currentChoro.cfbtwStartCell[0] && parseFloat(choroplethSelectedMeasureValue) <= currentChoro.cfbtwEndCell[0]) {
                        isCondition = true;
                        cfColor = currentChoro.color;
                      }

                      break;
                    case 'measure':
                      if (component.stateWiseAggregation[layerObj.name + choroLyr.name + 'StateAggregation'] && component.stateWiseAggregation[layerObj.name + choroLyr.name + 'StateAggregation'][tooltipValue]) {
                        if (parseFloat(choroplethSelectedMeasureValue) >= parseFloat(component.stateWiseAggregation[layerObj.name + choroLyr.name + 'StateAggregation'][tooltipValue][measure[currentChoro.cfbtwStartMsr]]) && parseFloat(choroplethSelectedMeasureValue) <= parseFloat(component.stateWiseAggregation[layerObj.name + choroLyr.name + 'StateAggregation'][tooltipValue][measure[currentChoro.cfbtwEndMsr]])) {
                          isCondition = true;
                          cfColor = currentChoro.color;
                        }
                      }

                      break;
                  }
                }
              }

              break;
            case 'measureCal':
              isCondition = false;
              if (component.stateWiseAggregation[layerObj.name + choroLyr.name + 'StateAggregation'] && component.stateWiseAggregation[layerObj.name + choroLyr.name + 'StateAggregation'][tooltipValue]){
              isCondition = eval(parseFloat(choroplethSelectedMeasureValue)            + currentChoro.operator            + (parseFloat(component.stateWiseAggregation[layerObj.name + choroLyr.name + 'StateAggregation'][tooltipValue][measure[currentChoro.calM1]])            + currentChoro.calOpr            + parseFloat(component.stateWiseAggregation[layerObj.name + choroLyr.name + 'StateAggregation'][tooltipValue][measure[currentChoro.calM2]]))); // jshint ignore:line
			  } 
              cfColor = currentChoro.color;
              break;
            case 'targetVal':
              var tgtRl = currentChoro.targetRule;
              var targetValue;
              switch (currentChoro.tValueType){
                case 'static':
                  targetValue = currentChoro.tStatic;
                  break;
                case 'dynamic':
                  if (currentChoro.tDynamicSelection == 'cell') {
                    targetValue = currentChoro.tDynamic[0];
                  }else {
                    if (undefined === component.stateWiseAggregation[layerObj.name + choroLyr.name + 'StateAggregation'][tooltipValue]){
                    targetValue = 0;}
                    if (component.stateWiseAggregation[layerObj.name + choroLyr.name + 'StateAggregation'] && component.stateWiseAggregation[layerObj.name + choroLyr.name + 'StateAggregation'][tooltipValue]){
                    targetValue = component.stateWiseAggregation[layerObj.name + choroLyr.name + 'StateAggregation'][tooltipValue][measure[currentChoro.tDynamicMeasure]];}
                    else{
                    targetValue = 0;}
                  }

                  break;
              }

              for (var i = 1; i <= Object.keys(tgtRl).length; i++) {
                if (choroplethSelectedMeasureValue && (choroplethSelectedMeasureValue >= ((tgtRl[i].from / 100) * targetValue)) && (choroplethSelectedMeasureValue <= ((tgtRl[i].to / 100) * targetValue))) {
                  isCondition = true;
                  cfColor = tgtRl[i].color;
                  break;
                }
              }

              break;
          }

          //If Icon type is only standard I am changing the markerIcon object.
          if (isCondition) {
            layer.setStyle({fillColor: cfColor});
          }
        }
      }
    }
  }
};
/**
 * The function will add legend to the map based on no. of layer's created.
 *
 * @return {null} .
 */
Choropleth.prototype.addLegend = function(measure, layerObj, choroLyr) {
  var self = this;
  var component = self.getProps();
  //Get the choropleth objects from additional properties.
  //var choroplethLayer = JSON.parse(component.choroplethLayers());

  if (choroLyr.geoLayer == layerObj.name) {
    var quantileColor = [];
    for (var key in choroLyr.quantile) {
      quantileColor.push(choroLyr.quantile[key].color);
    }

    var legendPosition =  'bottomright';

    if (choroLyr.legendPosition) {
      legendPosition = choroLyr.legendPosition;
    }

    var legend = L.control({position: legendPosition});
    component.choroplethLegend.push(legend);
    legend.onAdd = function() {
        var qlqz = choroLyr.qlOrQz;
        var quantileScales;
        //The if condition is just to mask the buckets.  I know there is some confusion.
        if (qlqz == 'quantile' || qlqz == 'quantize' || qlqz == 'linear') {
          quantileScales = 'quantile';
        } else if (qlqz == 'custom') {
          quantileScales = 'custom';
        }

        switch (quantileScales) {
        case 'quantile':
          var msrArray = self.unique(component.measureArray[measure[choroLyr.measure]]).sort(function(a, b) {
            return a - b;
          });

          var colorScale;
          if (choroLyr.nature != 'stan' && choroLyr.colorChoice == 'brewer') {
            colorScale = chroma.scale(choroLyr.brewerColor).colors(_.parseInt(choroLyr.buckets));
          }else if (choroLyr.nature == 'stan' && choroLyr.colorChoice == 'brewer') {
            colorScale = chroma.scale(component.colorBrewer[choroLyr.brewerColor]).colors(_.parseInt(choroLyr.buckets));
          }else {
            colorScale = chroma.scale(quantileColor).colors(_.parseInt(quantileColor.length));
          }

          var scale;
          if (qlqz == 'quantile') {
            //Calling d3 for Quantile Scales;
            scale = d3.scale.quantile().domain(msrArray).range(colorScale);
          } else if (qlqz == 'quantize') {
            //Calling d3 for Quantize Scales;
            scale = d3.scale.quantize().domain(msrArray).range(colorScale);
          } else if (qlqz == 'linear') {
            //Calling d3 for linear Scales;
            scale = d3.scale.linear().domain(msrArray).range(colorScale);
          }

          var numberOfRowsInLegend = colorScale.length;
          var count = Math.ceil(msrArray.length / numberOfRowsInLegend);

          var intervals = [];
          for (var i = 0; i < msrArray.length; i = i + count) {
            if (intervals.length >= numberOfRowsInLegend) {
              break;
            }

            intervals.push(msrArray[i]);
          }
          var div = L.DomUtil.create('div', 'container info legend legend-style'), grades = intervals;
          div.id = choroLyr.name + '-cho';
          
          
          var x = 0;
          var labels = '<b>' + choroLyr.name  + '</b> - ('+measure[choroLyr.measure]+')<br/>';
          labels = labels+'<div style="text-align:center;">'+grades[0].toFixed(2)+' to '+grades[grades.length-1].toFixed(2)+'</div>';
          var svg = '<svg width="' + grades.length * 15 + '" height="15">';
          for (var i = 0; i < grades.length; i++) {
            svg += '<rect fill="' + scale(grades[i]) + '" width="15" height="15" x="' + x + '"><title>'+grades[i].toFixed(2)+'</title></rect>';
            x = x + 15;
          }
          var containerDiv = '<div class="header1"><span><i class="fa fa-caret-square-o-right fa-2" aria-hidden="true"></i>&nbsp;&nbsp;Legend</span> </div><div class="content">'+labels+svg+'</div>';
          div.innerHTML = containerDiv;



          /*for (var i = 0; i < grades.length; i++) {
            if (i + 1 < grades.length) {
              labels = labels + '<tr><td><div style="border-radius: 25px;float:right;width:20px;height:20px;background-color:' + scale(grades[i]) + '"></div></td><td>' + grades[i].toFixed(2) + '</td><td>to</td><td>' + grades[i + 1].toFixed(2) + '</td></tr>';
            } else {
              labels = labels + '<tr><td><div style="border-radius: 25px;float:right;width:20px;height:20px;background-color:' + scale(grades[i]) + '"></div></td><td>' + grades[i].toFixed(2) + '</td><td>+</td><td></td></tr>';
            }
          }

          div.innerHTML = labels + '</table>';*/
          return div;
          
        case 'custom':
          var div = L.DomUtil.create('div', 'container info legend legend-style'), grades = intervals;
          div.id = choroLyr.name + '-cho';
          var customColor = choroLyr.quantize;
          var colorLength = Object.keys(customColor).length;
          var x = 0;
          var labels = '<div id="'+choroLyr.name+'" style="cursor:pointer"><b>' + choroLyr.name  + '</b> - ('+measure[choroLyr.measure]+')</div>';
          labels = labels+'<div style="text-align:center;">'+customColor[0].stop+' to '+customColor[colorLength-1].stop+'</div>';
          var svg = '<svg width="' + colorLength * 15 + '" height="15">';
          for (var key in customColor) {
            svg += '<rect fill="' + customColor[key].color + '" width="15" height="15" x="' + x + '"><title>'+customColor[key].stop+'</title></rect>';
            x = x + 15;
          }

          var containerDiv = '<div class="header1"><span><i class="fa fa-caret-square-o-right fa-2" aria-hidden="true"></i>&nbsp;&nbsp;Legend</span> </div><div class="content">'+labels+svg+'</div>';
          div.innerHTML = containerDiv;
          
          /*for (var key in customColor) {
            labels = labels + '<tr><td><div style="border-radius: 25px;float:right;width:20px;height:20px;background-color:' + customColor[key].color + '"></div></td><td>' + customColor[key].start + '</td><td>to</td><td>' + customColor[key].stop + '</td></tr>';
          }

          div.innerHTML = labels + '</table>';*/
          return div;
          
        }
      };

    legend.addTo(self.getMapContainer());
    $(".header1").off('click');
    $(".header1").on('click',function () {
            
        var $header = $(this);
        //getting the next element
        var $content = $header.next();
        //open up the content needed - toggle the slide- if visible, slide up, if not slidedown.
        $content.slideToggle(500, function () {
            //execute this after slideToggle is done
            //change text of header based on visibility of content div
            $header.html(function () {
                //change text based on condition
                return $content.is(":visible") ? '<span><i class="fa fa-caret-square-o-down fa-2" aria-hidden="true"></i>&nbsp;&nbsp;Legend</span>' : '<span><i class="fa fa-caret-square-o-right fa-2" aria-hidden="true"></i>&nbsp;&nbsp;Legend</span>';
            });
        });
    });
  }
};

Choropleth.prototype.unique = function(origArr) {
  var newArr = [], origLen = origArr.length, found, x, y;
  for (x = 0; x < origLen; x++) {
    found = undefined;
    for (y = 0; y < newArr.length; y++) {
      if (origArr[x] === newArr[y]) {
        found = true;
        break;
      }
    }

    if (!found) {
      newArr.push(origArr[x]);
    }
  }

  return newArr;
};
/**
 * The function will aggregate all the values and will say the minimum and maximum value for the measure.
 *
 * @return {null} .
 */
Choropleth.prototype.getMaxMin = function(dsRows, measure, layerObj, choroLyr) {
  var self = this;
  var component = self.getProps();
  //var dimension = layerObj.dimension;
  //If the global object value has the value then return it.  Else calculate it.
  if (component.stateWiseAggregation[layerObj.name + choroLyr.name]) {
    return component.stateWiseAggregation[layerObj.name + choroLyr.name];
  } else {
    //Statewise aggregation are stored in this object
    var stateAggregation = {};
    //Iterating the datasource to store the aggregated values
    for (var key in dsRows) {
      if (choroLyr.fDimension == 'No Filter' || $.inArray(dsRows[key][layerObj.dimension], choroLyr.filter) > -1) {
        var tempObj = {};
        var isResult = false;
        //Finding whether it's a result row.
        for (var res in dsRows[key]) {
          if (dsRows[key][res] == 'Result' || dsRows[key][res] == 'Overall Result') {
            isResult = true;
            break;
          }
        }
        //If it's a result row omitting it.
        if (!isResult) {
          for (var m in measure) {
            tempObj[measure[m]] = dsRows[key][m];
          }

          if (undefined === stateAggregation[dsRows[key][layerObj.dimension]]) {
            stateAggregation[dsRows[key][layerObj.dimension]] = tempObj;
          } else {
            for (var m in measure) {
              stateAggregation[dsRows[key][layerObj.dimension]][measure[m]] = stateAggregation[dsRows[key][layerObj.dimension]][measure[m]] + dsRows[key][m];
            }
          }
        }
      }
    }

    component.stateWiseAggregation[layerObj.name + choroLyr.name + 'StateAggregation'] = stateAggregation;
    //A object where it will store all the values, which it will be useful to find the min and max.
    var measureValue = {};
    //Initializing the array
    for (var m in measure) {
      measureValue[measure[m]] = [];
    }
    //Pushing all the values in the 'measureValue' object.
    for (var key in stateAggregation) {
      for (var m in measure) {
        measureValue[measure[m]].push(stateAggregation[key][measure[m]]);
      }
    }
    //Pusing all the object with respect to the geojson layer name.
    component.stateWiseAggregation[layerObj.name + choroLyr.name] = measureValue;
    return component.stateWiseAggregation[layerObj.name + choroLyr.name];
  }
};

Choropleth.prototype.rgbToHex = function(r, g, b) {
  return '#' + ((1 << 24) | (r << 16) | (g << 8) | b).toString(16).slice(1);
};
