/**
 * Creates an instance of MarkerLayer.
 *
 * @author Vignesh Gopalakrishnan
 * @constructor
 * @param {map, props} Map Instance and Addt prop object..
 * @this {MarkerLayer}
 */
/* global OverlappingMarkerSpiderfier */
/* global Scripting */
/* global LocDS */
/* global L */
/* global d3 */
/* global chroma */
function MarkerLayer(map, props) {
  var mapContainer = map;
  var addProps = props;
  var groupOfMarker = null;
  var util = new VbiUtil();
  var clusterGroupMarker = null;

  this.getClusterGroupOfMarker = function() {
    return clusterGroupMarker;
  };

  this.setClusterGroupOfMarker = function(clusterGroup) {
    this.clusterGroupMarker = clusterGroup;
  };

  this.getMapContainer = function() {
    return mapContainer;
  };

  this.getProps = function() {
    return addProps;
  };

  this.getGroupOfMarker = function() {
    return groupOfMarker;
  };

  this.setGroupOfMarker = function(markers) {
    this.groupOfMarker = markers;
  };

  this.getUtil = function() {
    return util;
  };
}

MarkerLayer.prototype.createLegend = function(component) {
  if (!$('.legend-marker').length) {
    //Collapsible legends
    if (component.legend !== null) {
      component.map.removeControl(component.legend);
    }
    component.legend = L.control();
    component.legend.onAdd = function() {
      this._div = L.DomUtil.create('div', 'legend-marker legend-style'); // create a div with a class "info"
      // this.update();
      return this._div;
    };
    component.legend.addTo(component.map);
    //$('#legendBox').css('height',component.height()/5+'px');
    component.legend.setPosition(component.cMarkerLegendPosition());
    $('.legend-marker').html('<div id="legendBox" style="max-height:200px; background-color: transperant;overflow: auto; display:none;"></div>');
    var legendContainer = '<div class="container"><div class="header"><span><i class="fa fa-caret-square-o-right fa-2" aria-hidden="true"></i>&nbsp;&nbsp;Legend</span> </div><div class="content" id="markerLegends"></div></div>';
    $('#legendBox').html(legendContainer);

    $(".header").click(function() {
      var $header = $(this);
      //getting the next element
      var $content = $header.next();
      //open up the content needed - toggle the slide- if visible, slide up, if not slidedown.
      $content.slideToggle(500, function() {
        //execute this after slideToggle is done
        //change text of header based on visibility of content div
        $header.html(function() {
          //change text based on condition
          return $content.is(":visible") ? '<span><i class="fa fa-caret-square-o-down fa-2" aria-hidden="true"></i>&nbsp;&nbsp;Legend</span>' : '<span><i class="fa fa-caret-square-o-right fa-2" aria-hidden="true"></i>&nbsp;&nbsp;Legend</span>';
        });
      });
    });
  }
};

MarkerLayer.prototype.iteratePerDataProvider = function(self, component, z) {
  var currentResultSet;
  var dataConfigs;
  if (component.isDataProvider() && component.locDatasourceProperties()) {
    dataConfigs = JSON.parse(component.locDatasourceProperties());
  }
  var dataProviderID = component.dataProviderArray[z];
  //If user enabled multiple data then loop will enter this if loop
  if (dataProviderID != 'VBX_DEFAULT_DS') {
    currentResultSet = JSON.parse(component.getDataProviderResult(dataProviderID));
  } else {
    //Checking if the resultset is there.  else breaking the whole operation.
    if (component.resultset()) {
      currentResultSet = component.resultset();
    } else {
      currentResultSet = undefined;
      self.removeAllLayers();
      return currentResultSet;
    }
  }
  if (currentResultSet !== undefined) {
    var lyrObjLayers = self.renderLayer(currentResultSet, dataProviderID, dataConfigs);
    if (component.browser) {
      setTimeout(function() {
        for (var lyr in lyrObjLayers) {
          self.addLegend(self, component, lyrObjLayers, lyr);
        }
      }, 2000);
    }
  }
  return "defined";
};

MarkerLayer.prototype.deleteLayer = function(layer) {
  var self = this;
  var component = self.getProps();
  if (layer) {
    var id = L.Util.stamp(layer);
    var obj = self.getMapContainer()._layers[id];
    if (obj && self.getMapContainer().hasLayer(obj)) {
      self.getMapContainer().removeLayer(obj);
      component.layerControl.removeLayer(layer);
      delete self.getMapContainer()._layers[id];
    }
  }
};

MarkerLayer.prototype.createMarkerLayerType = function(component, layers, self) {
  //The below three consecutive if conditions will get all the layers from addt props and add it to layer array.
  //This is for Marker Layer
  if (typeof component.useMultipleMarker() == 'boolean' && component.useMultipleMarker() && component.mMarkerLayerAdd() && Object.keys(JSON.parse(component.mMarkerLayerAdd())).length > 0) {
    var markerLayer = JSON.parse(component.mMarkerLayerAdd());
    for (var ml in markerLayer) {
      if (markerLayer[ml].isDataProvider === undefined) {
        markerLayer[ml].isDataProvider = false;
      }
      if (markerLayer[ml].clusterLimit === undefined) {
        markerLayer[ml].clusterLimit = 10000;
      }
      /* if(markerLayer[ml].selectedColor==='rgba(255, 255, 255, 0)'){
         markerLayer[ml].selectedColor='transperant';
       }*/
      if (!component.isDataProvider() && markerLayer[ml].isDataProvider) {
        self.deleteLayer(component.markerReference[markerLayer[ml].layer + '|' + markerLayer[ml].name]);
        delete component.markerReference[markerLayer[ml].layer + '|' + markerLayer[ml].name];
        delete markerLayer[ml];
      } else if (component.isDataProvider() && !markerLayer[ml].isDataProvider) {
        self.deleteLayer(component.markerReference[markerLayer[ml].layer + '|' + markerLayer[ml].name]);
        delete component.markerReference[markerLayer[ml].layer + '|' + markerLayer[ml].name];
        delete markerLayer[ml];
      }

      if (markerLayer[ml] && component.backUpMarkerLayer && (JSON.stringify(markerLayer[ml]) == JSON.stringify(component.backUpMarkerLayer[ml]))) {
        if (component.markerReference[markerLayer[ml].layer + '|' + markerLayer[ml].name] !== undefined) {
          delete markerLayer[ml];
        }
      } else if (markerLayer[ml] && component.backUpMarkerLayer && (JSON.stringify(markerLayer[ml]) != JSON.stringify(component.backUpMarkerLayer[ml]))) {
        self.deleteLayer(component.markerReference[markerLayer[ml].layer + '|' + markerLayer[ml].name]);
        delete component.markerReference[markerLayer[ml].layer + '|' + markerLayer[ml].name];
      }
    }
    
    if (Object.keys(markerLayer).length > 0) {
      if (!component.backUpMarkerLayer) {
        component.backUpMarkerLayer = markerLayer;
        layers.push(markerLayer);
      } else if (JSON.stringify(component.backUpMarkerLayer) == JSON.stringify(markerLayer)) {
        //do nothing - knowing left this else if blank.
      } else if (JSON.stringify(component.backUpMarkerLayer) != JSON.stringify(markerLayer)) {
        component.backUpMarkerLayer = markerLayer;
        layers.push(markerLayer);
      }
    }
  } else if (component.mMarkerLayerAdd() && Object.keys(JSON.parse(component.mMarkerLayerAdd())).length > 0) {
    var markerLayer = JSON.parse(component.mMarkerLayerAdd());
    //delete all layers.
    if (Object.keys(markerLayer).length > 0) {
      for (var ml in markerLayer) {
        self.deleteLayer(component.markerReference[markerLayer[ml].layer + '|' + markerLayer[ml].name]);
        delete component.markerReference[markerLayer[ml].layer + '|' + markerLayer[ml].name];
      }
      component.backUpMarkerLayer = null;
    }
  } else if (component.backUpMarkerLayer && Object.keys(JSON.parse(component.mMarkerLayerAdd())).length == 0) {
    var markerLayer = component.backUpMarkerLayer;
    //delete all layers.
    if (Object.keys(markerLayer).length > 0) {
      for (var ml in markerLayer) {
        self.deleteLayer(component.markerReference[markerLayer[ml].layer + '|' + markerLayer[ml].name]);
        delete component.markerReference[markerLayer[ml].layer + '|' + markerLayer[ml].name];
      }
      component.backUpMarkerLayer = null;
    }
  }
};

MarkerLayer.prototype.addConditionalFormattingToLayers = function(conditionalFormatting, layers) {
  //Iterating the conditional formatting rule and adding it to the layer object.
  if (conditionalFormatting) {
    for (var rule in conditionalFormatting) {
      var layerName, layerType;
      if (conditionalFormatting[rule].newAPS) {
        layerName = conditionalFormatting[rule].mapLayer;
        layerType = conditionalFormatting[rule].layerType;
      } else {
        layerName = conditionalFormatting[rule].mapLayer.split('-')[0];
        layerType = conditionalFormatting[rule].mapLayer.split('-')[1];
      }

      for (var lyr in layers) {
        if (layers[lyr].cf === undefined) {
          layers[lyr].cf = [];
        }
        //If the layer type matches then appending the conditional formatting object to it.
        if (layers[lyr].layer == layerType && layers[lyr].name == layerName) {
          layers[lyr].cf.push(conditionalFormatting[rule]);
        }
      }
    }
  }
  return layers;
};

MarkerLayer.prototype.createClusterLayerType = function(component, layers, self) {
  //This is for Cluster Layer
  if (typeof component.useCluster() == 'boolean' && component.useCluster() && component.clusterLayerAdd() && Object.keys(JSON.parse(component.clusterLayerAdd())).length > 0) {
    //Fix for migration regarding object name
    var clusterLayers = JSON.parse(component.clusterLayerAdd());
    for (var lyr in clusterLayers) {
      if (clusterLayers[lyr].dimension) {
        clusterLayers[lyr].fDimension = clusterLayers[lyr].dimension;
      }
      if (clusterLayers[lyr].sFilter) {
        clusterLayers[lyr].filter = clusterLayers[lyr].sFilter;
      }
      if (clusterLayers[lyr].isDataProvider === undefined) {
        clusterLayers[lyr].isDataProvider = false;
      }

      if (!component.isDataProvider() && clusterLayers[lyr].isDataProvider) {
        self.deleteLayer(component.markerReference[clusterLayers[lyr].layer + '|' + clusterLayers[lyr].name]);
        delete component.markerReference[clusterLayers[lyr].layer + '|' + clusterLayers[lyr].name];
        delete clusterLayers[lyr];
      } else if (component.isDataProvider() && !clusterLayers[lyr].isDataProvider) {
        self.deleteLayer(component.markerReference[clusterLayers[lyr].layer + '|' + clusterLayers[lyr].name]);
        delete component.markerReference[clusterLayers[lyr].layer + '|' + clusterLayers[lyr].name];
        delete clusterLayers[lyr];
      }

      if (clusterLayers[lyr] && component.backUpClusterLayer && (JSON.stringify(clusterLayers[lyr]) == JSON.stringify(component.backUpClusterLayer[lyr]))) {
        if (component.markerReference[clusterLayers[lyr].layer + '|' + clusterLayers[lyr].name] !== undefined) {
          delete clusterLayers[lyr];
        }
      } else if (clusterLayers[lyr] && component.backUpClusterLayer && (JSON.stringify(clusterLayers[lyr]) != JSON.stringify(component.backUpClusterLayer[lyr]))) {
        self.deleteLayer(component.markerReference[clusterLayers[lyr].layer + '|' + clusterLayers[lyr].name]);
        delete component.markerReference[clusterLayers[lyr].layer + '|' + clusterLayers[lyr].name];
      }
    }
    if (Object.keys(clusterLayers).length > 0) {
      if (!component.backUpClusterLayer) {
        component.backUpClusterLayer = clusterLayers;
        layers.push(clusterLayers);
      } else if (JSON.stringify(component.backUpClusterLayer) == JSON.stringify(clusterLayers)) {
        //do nothing - knowing left this else if blank.
      } else if (JSON.stringify(component.backUpClusterLayer) != JSON.stringify(clusterLayers)) {
        component.backUpClusterLayer = clusterLayers;
        layers.push(clusterLayers);
      }
    }
  } else if (component.clusterLayerAdd() && Object.keys(JSON.parse(component.clusterLayerAdd())).length > 0) {
    var clusterLayers = JSON.parse(component.clusterLayerAdd());
    //delete all layers.
    if (Object.keys(clusterLayers).length > 0) {
      for (var ml in clusterLayers) {
        self.deleteLayer(component.markerReference[clusterLayers[ml].layer + '|' + clusterLayers[ml].name]);
        delete component.markerReference[clusterLayers[ml].layer + '|' + clusterLayers[ml].name];
      }
      component.backUpClusterLayer = null;
    }
  } else if (component.backUpClusterLayer && Object.keys(JSON.parse(component.clusterLayerAdd())).length == 0) {
    var clusterLayers = component.backUpClusterLayer;
    //delete all layers.    
    if (Object.keys(clusterLayers).length > 0) {
      for (var ml in clusterLayers) {
        self.deleteLayer(component.markerReference[clusterLayers[ml].layer + '|' + clusterLayers[ml].name]);
        delete component.markerReference[clusterLayers[ml].layer + '|' + clusterLayers[ml].name];
      }
      component.backUpClusterLayer = null;
    }
  }
};

MarkerLayer.prototype.createHeatLayerType = function(component, layers, self) {
  //This is for HeatMap Layer.
  if (component.useHeatMap() && component.markerLayer() && Object.keys(JSON.parse(component.markerLayer())).length > 0) {
    //Fix more migration issue from old bin to new one.
    var heatLayers = JSON.parse(component.markerLayer());
    for (var lyr in heatLayers) {
      //Suspecting an issue at any point of time in the below two if conditions.
      if (undefined === heatLayers[lyr].layer && heatLayers[lyr].filter) {
        heatLayers[lyr].fDimension = heatLayers[lyr].filter;
      }
      if (undefined === heatLayers[lyr].layer && heatLayers[lyr].sFilter) {
        heatLayers[lyr].filter = heatLayers[lyr].sFilter;
      }
      if (heatLayers[lyr].isDataProvider === undefined) {
        heatLayers[lyr].isDataProvider = false;
      }
      if (heatLayers[lyr].isColorScheme === undefined) {
        heatLayers[lyr].isColorScheme = false;
      }
      //Introduced a new variable to support old objects.
      heatLayers[lyr].layer = 'heat';

      if (!component.isDataProvider() && heatLayers[lyr].isDataProvider) {
        self.deleteLayer(component.markerReference[heatLayers[lyr].layer + '|' + heatLayers[lyr].name]);
        delete component.markerReference[heatLayers[lyr].layer + '|' + heatLayers[lyr].name];
        delete heatLayers[lyr];
      } else if (component.isDataProvider() && !heatLayers[lyr].isDataProvider) {
        self.deleteLayer(component.markerReference[heatLayers[lyr].layer + '|' + heatLayers[lyr].name]);
        delete component.markerReference[heatLayers[lyr].layer + '|' + heatLayers[lyr].name];
        delete heatLayers[lyr];
      }

      if (heatLayers[lyr] && component.backUpHeatLayer && (JSON.stringify(heatLayers[lyr]) == JSON.stringify(component.backUpHeatLayer[lyr]))) {
        if (component.markerReference[heatLayers[lyr].layer + '|' + heatLayers[lyr].name] !== undefined) {
          delete heatLayers[lyr];
        }
      } else if (heatLayers[lyr] && component.backUpHeatLayer && (JSON.stringify(heatLayers[lyr]) != JSON.stringify(component.backUpHeatLayer[lyr]))) {
        self.deleteLayer(component.markerReference[heatLayers[lyr].layer + '|' + heatLayers[lyr].name]);
        delete component.markerReference[heatLayers[lyr].layer + '|' + heatLayers[lyr].name];
      }
    }

    if (Object.keys(heatLayers).length > 0) {
      if (!component.backUpHeatLayer) {
        component.backUpHeatLayer = heatLayers;
        layers.push(heatLayers);
      } else if (JSON.stringify(component.backUpHeatLayer) == JSON.stringify(heatLayers)) {
        //do nothing -laye knowing left this else if blank.
      } else if (JSON.stringify(component.backUpHeatLayer) != JSON.stringify(heatLayers)) {
        component.backUpHeatLayer = heatLayers;
        layers.push(heatLayers);
      }
    }
  } else if (component.markerLayer() && Object.keys(JSON.parse(component.markerLayer())).length > 0) {
    var heatLayers = JSON.parse(component.markerLayer());
    //delete all layers.
    if (Object.keys(heatLayers).length > 0) {
      for (var ml in heatLayers) {
        self.deleteLayer(component.markerReference[heatLayers[ml].layer + '|' + heatLayers[ml].name]);
        delete component.markerReference[heatLayers[ml].layer + '|' + heatLayers[ml].name];
      }
      component.backUpHeatLayer = null;
    }
  } else if (component.backUpHeatLayer && Object.keys(JSON.parse(component.markerLayer())).length == 0) {
    var heatLayers = component.backUpHeatLayer;
    //delete all layers.
    if (Object.keys(heatLayers).length > 0) {
      for (var ml in heatLayers) {
        self.deleteLayer(component.markerReference[heatLayers[ml].layer + '|' + heatLayers[ml].name]);
        delete component.markerReference[heatLayers[ml].layer + '|' + heatLayers[ml].name];

      }
      component.backUpHeatLayer = null;
    }
  }
};

MarkerLayer.prototype.createBubbleLayerType = function(component, layers, self) {
  //This is for bubble map
  if (component.useBubbleMap() && component.bubbleButton() && Object.keys(JSON.parse(component.bubbleButton())).length > 0) {
    var bubbleLayers = JSON.parse(component.bubbleButton());
    for (var lyr in bubbleLayers) {
      if (bubbleLayers[lyr].isDataProvider === undefined) {
        bubbleLayers[lyr].isDataProvider = false;
      }

      if (!component.isDataProvider() && bubbleLayers[lyr].isDataProvider) {
        self.deleteLayer(component.markerReference[bubbleLayers[lyr].layer + '|' + bubbleLayers[lyr].name]);
        delete component.markerReference[bubbleLayers[lyr].layer + '|' + bubbleLayers[lyr].name];
        delete bubbleLayers[lyr];
      } else if (component.isDataProvider() && !bubbleLayers[lyr].isDataProvider) {
        self.deleteLayer(component.markerReference[bubbleLayers[lyr].layer + '|' + bubbleLayers[lyr].name]);
        delete component.markerReference[bubbleLayers[lyr].layer + '|' + bubbleLayers[lyr].name];
        delete bubbleLayers[lyr];
      }

      if (bubbleLayers[lyr] && component.backUpBubbleLayer && (JSON.stringify(bubbleLayers[lyr]) == JSON.stringify(component.backUpBubbleLayer[lyr]))) {
        if (component.markerReference[bubbleLayers[lyr].layer + '|' + bubbleLayers[lyr].name] !== undefined) {
          delete bubbleLayers[lyr];
        }
      } else if (bubbleLayers[lyr] && component.backUpBubbleLayer && (JSON.stringify(bubbleLayers[lyr]) != JSON.stringify(component.backUpBubbleLayer[lyr]))) {
        self.deleteLayer(component.markerReference[bubbleLayers[lyr].layer + '|' + bubbleLayers[lyr].name]);
        delete component.markerReference[bubbleLayers[lyr].layer + '|' + bubbleLayers[lyr].name];
      }
    }

    if (Object.keys(bubbleLayers).length > 0) {
      if (!component.backUpBubbleLayer) {
        component.backUpBubbleLayer = bubbleLayers;
        layers.push(bubbleLayers);
      } else if (JSON.stringify(component.backUpBubbleLayer) == JSON.stringify(bubbleLayers)) {
        //do nothing - knowing left this else if blank.
      } else if (JSON.stringify(component.backUpBubbleLayer) != JSON.stringify(bubbleLayers)) {
        component.backUpBubbleLayer = bubbleLayers;
        layers.push(bubbleLayers);
      }
    }
  } else if (component.bubbleButton() && Object.keys(JSON.parse(component.bubbleButton())).length > 0) {
    var bubbleLayers = JSON.parse(component.bubbleButton());
    //delete all layers.
    if (Object.keys(bubbleLayers).length > 0) {
      for (var ml in bubbleLayers) {
        self.deleteLayer(component.markerReference[bubbleLayers[ml].layer + '|' + bubbleLayers[ml].name]);
        delete component.markerReference[bubbleLayers[ml].layer + '|' + bubbleLayers[ml].name];
      }
      component.backUpBubbleLayer = null;
    }
  } else if (component.backUpBubbleLayer && Object.keys(JSON.parse(component.bubbleButton())).length == 0) {
    var bubbleLayers = component.backUpBubbleLayer;
    //delete all layers.
    if (Object.keys(bubbleLayers).length > 0) {
      for (var ml in bubbleLayers) {
        self.deleteLayer(component.markerReference[bubbleLayers[ml].layer + '|' + bubbleLayers[ml].name]);
        delete component.markerReference[bubbleLayers[ml].layer + '|' + bubbleLayers[ml].name];
      }
      component.backUpBubbleLayer = null;
    }
  }
};

MarkerLayer.prototype.conditionalFormattingColor = function(cValue, cfColor, cfSymbol, cfSymCol, isCondition, isConditionMatched, layers, dsRows, cond, lyr, text, count) {
  if (isCondition) {
    isConditionMatched = true;
  }

  isCondition = false;
  var currentCond = layers[lyr].cf[cond];
  if(currentCond.newAPS){
    currentCond.color = currentCond.markerColor;
  }
  var layerName = currentCond.mapLayer.split('-')[0];
  var layerType = currentCond.mapLayer.split('-')[1];
  //Getting the comparison measure value
  var comparisonMeasure = dsRows[text[count]][currentCond.cMeasure];
  //This variable will change based on the comparison value type.

  if (currentCond.cValueType == 'static') {
    cValue = currentCond.cValue;
  } else if (currentCond.cValueType == 'dynamic') {
    if (currentCond.cDynamicType == 'cell') {
      cValue = currentCond.dValue[0];
    } else {
      cValue = dsRows[text[count]][currentCond.cDynamicMeasure];
    }
  }

  //Based on the rule type the different conditions will be executed.
  switch (currentCond.ruleType) {
    case 'single':
      //The operator user chooses is used here.
      //I know using eval is not right.  But no other go for me here to evaluavate this.  That's okay.
      if (currentCond.operator != 'btw') {
        isCondition = eval(parseFloat(comparisonMeasure) + currentCond.operator + parseFloat(cValue)); // jshint ignore:line
        cfColor = currentCond.color;
      } else {
        if (currentCond.cValueType == 'static') {
          isCondition = parseFloat(comparisonMeasure) >= currentCond.cfbtwStartStatic && parseFloat(comparisonMeasure) <= currentCond.cfbtwEndStatic;
          if (isCondition) {
            cfColor = currentCond.color;
          }
        } else if (currentCond.cValueType == 'dynamic') {
          switch (currentCond.cDynamicType) {
            case 'cell':
              isCondition = parseFloat(comparisonMeasure) >= currentCond.cfbtwStartCell[0] && parseFloat(comparisonMeasure) <= currentCond.cfbtwEndCell[0];
              if (isCondition) {
                cfColor = currentCond.color;
              }
              break;
            case 'measure':
              isCondition = parseFloat(comparisonMeasure) >= parseFloat(dsRows[text[count]][currentCond.cfbtwStartMsr]) && parseFloat(comparisonMeasure) <= parseFloat(dsRows[text[count]][currentCond.cfbtwEndMsr]);
              if (isCondition) {
                cfColor = currentCond.color;
              }
              break;
          }
        }
      }

      break;
    case 'measureCal':
      isCondition = eval(parseFloat(comparisonMeasure) + currentCond.operator + (parseFloat(dsRows[text[count]][currentCond.calM1]) + currentCond.calOpr + parseFloat(dsRows[text[count]][currentCond.calM2]))); // jshint ignore:line
      cfColor = currentCond.color;
      break;
    case 'targetVal':
      var tgtRl = currentCond.targetRule;
      var targetValue;
      switch (currentCond.tValueType) {
        case 'static':
          targetValue = currentCond.tStatic;
          break;
        case 'dynamic':
          if (currentCond.tDynamicSelection == 'cell') {
            targetValue = currentCond.tDynamic[0];
          } else {
            targetValue = dsRows[text[count]][currentCond.tDynamicMeasure];
          }
          break;
      }

      for (var i = 1; i <= Object.keys(tgtRl).length; i++) {
        if ((comparisonMeasure >= ((tgtRl[i].from / 100) * targetValue)) && (comparisonMeasure <= ((tgtRl[i].to / 100) * targetValue))) {
          isCondition = true;
          cfColor = tgtRl[i].color;
          if (layers[lyr].iconType == 'symbol') {
            cfSymbol = tgtRl[i].symbol;
            cfSymCol = tgtRl[i].symCol;
          }

          break;
        }
      }

      break;
    case 'dim':
      var cfDim = dsRows[text[count]][currentCond.dim];
      if (cfDim == currentCond.dimMember) {
        isCondition = true;
        cfColor = currentCond.color;
      }

      break;
  }
  return {
    'cValue': cValue,
    'cfColor': cfColor,
    'cfSymbol': cfSymbol,
    'cfSymCol': cfSymCol,
    'layerName': layerName,
    'currentCond': currentCond,
    'layerType': layerType,
    'isCondition': isCondition
  };
};

MarkerLayer.prototype.bubbleLayerConditionalFormatting = function(layers, dsRows, color, lyr, text, count) {
  //Below if is Completely for conditional formatting - If and only if the conditional formatting is undefined it will work.
  if (layers[lyr].cf) {
    //This variable will change based on the comparison value type.
    var cValue, isCondition, cfColor;

    for (var cond in layers[lyr].cf) {
      var currentCond = layers[lyr].cf[cond];
      //Getting the comparison measure value
      var comparisonMeasure = dsRows[text[count]][currentCond.cMeasure];

      if (currentCond.cValueType == 'static') {
        cValue = currentCond.cValue;
      } else if (currentCond.cValueType == 'dynamic') {
        if (currentCond.cDynamicType == 'cell') {
          cValue = currentCond.dValue[0];
        } else {
          cValue = dsRows[text[count]][currentCond.cDynamicMeasure];
        }
      }

      //Based on the rule type the different conditions will be executed.
      switch (currentCond.ruleType) {
        case 'single':
          //The operator user chooses is used here.
          //I know using eval is not right.  But no other go for me here to evaluavate this.  That's okay.
          if (currentCond.operator != 'btw') {
            isCondition = eval(parseFloat(comparisonMeasure) + currentCond.operator + parseFloat(cValue)); // jshint ignore:line
            cfColor = currentCond.color;
          } else {
            if (currentCond.cValueType == 'static') {
              if (parseFloat(comparisonMeasure) >= currentCond.cfbtwStartStatic && parseFloat(comparisonMeasure) <= currentCond.cfbtwEndStatic) {
                isCondition = true;
                cfColor = currentCond.color;
              }
            } else if (currentCond.cValueType == 'dynamic') {
              switch (currentCond.cDynamicType) {
                case 'cell':
                  if (parseFloat(comparisonMeasure) >= currentCond.cfbtwStartCell[0] && parseFloat(comparisonMeasure) <= currentCond.cfbtwEndCell[0]) {
                    isCondition = true;
                    cfColor = currentCond.color;
                  }

                  break;
                case 'measure':
                  if (parseFloat(comparisonMeasure) >= parseFloat(dsRows[text[count]][currentCond.cfbtwStartMsr]) && parseFloat(comparisonMeasure) <= parseFloat(dsRows[text[count]][currentCond.cfbtwEndMsr])) {
                    isCondition = true;
                    cfColor = currentCond.color;
                  }

                  break;
              }
            }
          }

          break;
        case 'measureCal':
          isCondition = eval(parseFloat(comparisonMeasure) + currentCond.operator + (parseFloat(dsRows[text[count]][currentCond.calM1]) + currentCond.calOpr + parseFloat(dsRows[text[count]][currentCond.calM2]))); // jshint ignore:line
          cfColor = currentCond.color;
          break;
        case 'targetVal':
          var tgtRl = currentCond.targetRule,
            tValue;
          switch (currentCond.tValueType) {
            case 'static':
              tValue = currentCond.tStatic;
              break;
            case 'dynamic':
              if (currentCond.tDynamicSelection == 'cell') {
                tValue = currentCond.tDynamic[0];
              } else {
                tValue = dsRows[text[count]][currentCond.tDynamicMeasure];
              }

              break;
          }
          for (var i = 1; i <= Object.keys(tgtRl).length; i++) {
            if ((comparisonMeasure >= ((tgtRl[i].from / 100) * tValue)) && (comparisonMeasure <= ((tgtRl[i].to / 100) * tValue))) {
              isCondition = true;
              cfColor = tgtRl[i].color;
              break;
            }
          }

          break;
        case 'dim':
          var cfDim = dsRows[text[count]][currentCond.dim];
          if (cfDim == currentCond.dimMember) {
            isCondition = true;
            cfColor = currentCond.color;
          }

          break;
      }

      //Changing the color of the bubble if the condition met.
      if (isCondition) {
        color = cfColor;
        isCondition = false;
      }
    }
  }
  return color;

};

MarkerLayer.prototype.addSpiderifyEvent = function(component, oms, scripting) {
  if (component.isSpiderifyEnabled() && component.browser) {
    //Adding this listener because of the marker should spiderify and click event should happen.
    //Issue - VBX141-397
    oms.addListener('spiderfy', function(markers) {
      component.isSpiderfied = true;
      for (var i = 0, len = markers.length; i < len; i++) {
        //Marker Onclick the following scripting function will be called.
        markers[i].on('click', scripting.callDSScripting);
      }
    });

    oms.addListener('unspiderfy', function(markers) {
      setTimeout(function() {
        component.isSpiderfied = false;
        for (var i = 0, len = markers.length; i < len; i++) {
          //Marker Onclick the following scripting function will be called.
          markers[i].off('click', scripting.callDSScripting);
        }
      }, 1000);
    });
  }
};

MarkerLayer.prototype.addLegend = function(self, component, layers, lyr) {

  if (component.mapLegendTracker[layers[lyr].name] === undefined || component.mapLegendTracker[layers[lyr].name] != layers[lyr].layer) {
    $('#legendBox').show();
    //Adding Legend
    var legendTable = '<table style="cursor:pointer">';
    if (layers[lyr].iconType === 'standard' || layers[lyr].iconType === 'font') {
      legendTable = legendTable + '<tr class="lName" id=' + layers[lyr].name + '>' +
        '<td width="40px" align=center><div style="border-radius: 50%; width: 20px; height: 20px; background-color:' + layers[lyr].color + '"></div></td>' +
        '<td width="60px" align=center>' + layers[lyr].name + '</td></tr>';
    } else if (layers[lyr].iconType === 'symbol') {
      legendTable = legendTable + '<tr class="lName" id=' + layers[lyr].name + '>' +
        '<td width="40px" align=center><i class="fa fa-' + layers[lyr].symbol + '" style="color:' + layers[lyr].symbolColor + '"></i></td>' +
        '<td width="60px" align=center>' + layers[lyr].name + '</td></tr>';
    } else if (layers[lyr].iconType === 'url') {
      legendTable = legendTable + '<tr class="lName" id=' + layers[lyr].name + '>' +
        '<td width="40px" align=center><img height="20px" width="20px" src="' + layers[lyr].url + '"/></td>' +
        '<td width="60px" align=center>' + layers[lyr].name + '</td></tr>';
    }
    legendTable = legendTable + '</table>';
    $('#markerLegends').append(legendTable);

    $(".lName").off('click', function() {});
    $(".lName").on('click', function(event) {
      var layerControl = {};
      //Separating and organizing the layers
      for (var lyr in component.layerControl._layers) {
        layerControl[component.layerControl._layers[lyr].name] = component.layerControl._layers[lyr].layer;
      }
      event.stopImmediatePropagation();
      if (self.getMapContainer().hasLayer(layerControl[this.id])) {
        $('#' + this.id).css('background-color', 'lightgray');
        $('#' + this.id).css('opacity', '0.4');
        $(this).removeClass('selected');
        self.getMapContainer().removeLayer(layerControl[this.id]);
      } else {
        $('#' + this.id).css('background-color', 'transparent');
        $('#' + this.id).css('opacity', '1');
        self.getMapContainer().addLayer(layerControl[this.id]);
        $(this).addClass('selected');
      }
    });
    component.mapLegendTracker[layers[lyr].name] = layers[lyr].layer;
  }
};

MarkerLayer.prototype.plotAddressGeoCoding = function(self, component, dsRowsKey, dsRows, lat, lng, address, dataArr, text, count, addressIndexes) {

  try {
    address = self.getAddress(dsRowsKey[text[count]], dsRows[text[count]], addressIndexes);
    dataArr = self.getUtil().getLatLng(address, component.googleGeoKey());
    if (dataArr.status == 'OK') {
      lat = dataArr.results[0].geometry.location.lat;
      lng = dataArr.results[0].geometry.location.lng;
      return {
        'lat': lat,
        'lng': lng
      };
    } else {
      return true;
    }

  } catch (e) {
    //Possible situtations.
    //1.  Might be the data returned from google api doesn't suits our code.
    if (window.console) {
      console.log(e); // jshint ignore:line
    }
  }
};

MarkerLayer.prototype.renderLayer = function(currentResultSet, dataProviderID, dataConfigs) {
  var self = this;
  var component = self.getProps();
  var scripting = new Scripting();
  var layers = [];
  var aggregateArray = [];
  var vbiCharts = new VbiCharts(component);
  var datasource = LocDS.getInstance();
  var dsRows = datasource.getDMKV(currentResultSet, 'text');
  var dsRowsKey = datasource.getDMKV(currentResultSet, 'key');
  var dsTextName = datasource.getDimensionAndMeasureObj(currentResultSet);
  var dsDimension = datasource.getDimensionObj(currentResultSet);
  var dsMeasure = datasource.getMeasureName(currentResultSet);
  var measures = datasource.getMeasureName(currentResultSet, 'text');
  var measureUnit = datasource.getMeasureUnit(currentResultSet);
  var scalingFactor = datasource.getScalingFactor(currentResultSet);


  var sampleCount, soloMarker, tootlTipText, markerIcon, bubbleMarker;
  //var choropleth = new Choropleth(self.getMapContainer(), component);
  var util = new VbiUtil();
  var oms = new OverlappingMarkerSpiderfier(self.getMapContainer());

  if (component.isSpiderfied) {
    component.$().click();
  }
  var conditionalFormatting;
  if (component.cfButton()) {
    conditionalFormatting = JSON.parse(component.cfButton());
  }

  self.getMapContainer().removeLayer(self.getMapContainer()._layers);
  self.createMarkerLayerType(component, layers, self);
  self.createClusterLayerType(component, layers, self);
  self.createHeatLayerType(component, layers, self);
  self.createBubbleLayerType(component, layers, self);
  //Concating all the marker and cluster layers together.
  layers = self.getUtil().concatArrayOfObjects(layers);

  //sample count
  if (component.samplePreview() && typeof component.noOfRowsToDisplay() == 'number' && component.noOfRowsToDisplay() !== 0) {
    sampleCount = component.noOfRowsToDisplay();
  }
  //Change Key-Text issue - Migration fix.
  for (var lyr in layers) {
    if (dsTextName[layers[lyr].fDimension] === undefined) {
      for (var ky in dsTextName) {
        if (dsTextName[ky] == layers[lyr].fDimension) {
          layers[lyr].fDimension = ky;
        }
      }
    }
  }

  layers = self.addConditionalFormattingToLayers(conditionalFormatting, layers);
  if (component.browser) {
    self.addSpiderifyEvent(component, oms, scripting);
  }
  //Iterating the layers objects which we get from additional properties. - Marker Layer
  for (var lyr in layers) {
    var latitudeIndex, longitudeIndex, toolTipIndex;
    var noOfMarkers = 0;
    var addressIndexes = {};
    if (component.isDataProvider() && layers[lyr].isDataProvider) {
      if (layers[lyr].dataSource && layers[lyr].dataSource !== dataProviderID) {
        continue;
      } else {
        latitudeIndex = dataConfigs[dataProviderID].latitudeIndex;
        longitudeIndex = dataConfigs[dataProviderID].longitudeIndex;
        toolTipIndex = dataConfigs[dataProviderID].ttipIndex;

        addressIndexes['countryIndex'] = dataConfigs[dataProviderID].countryIndex;
        addressIndexes['countrySwitch'] = dataConfigs[dataProviderID].countrySwitch;
        addressIndexes['stateIndex'] = dataConfigs[dataProviderID].stateIndex;
        addressIndexes['stateSwitch'] = dataConfigs[dataProviderID].stateSwitch;
        addressIndexes['cityIndex'] = dataConfigs[dataProviderID].cityIndex;
        addressIndexes['citySwitch'] = dataConfigs[dataProviderID].citySwitch;
        addressIndexes['zipIndex'] = dataConfigs[dataProviderID].zipIndex;
        addressIndexes['zipSwitch'] = dataConfigs[dataProviderID].zipSwitch;

        component.datasourceUsed.push(dataProviderID);
      }
    } else if (!layers[lyr].isDataProvider && !component.isDataProvider()) {
      latitudeIndex = component.latitudeIndex();
      longitudeIndex = component.longitudeIndex();
      toolTipIndex = component.ttipIndex();

      addressIndexes['countryIndex'] = component.countryIndex();
      addressIndexes['countrySwitch'] = component.countrySwitch();
      addressIndexes['stateIndex'] = component.stateIndex();
      addressIndexes['stateSwitch'] = component.stateSwitch();
      addressIndexes['cityIndex'] = component.cityIndex();
      addressIndexes['citySwitch'] = component.citySwitch();
      addressIndexes['zipIndex'] = component.zipIndex();
      addressIndexes['zipSwitch'] = component.zipSwitch();
    } else {
      continue;
    }

    //If the multiple data is disabled not showing those layers here.
    if (layers[lyr].isDataProvider && !component.isDataProvider()) {
      continue;
    }
    //Gets marker object based on the layer object.
    markerIcon = self.getMarker(layers[lyr]);
    //Filtering Values based on dimension.
    var filter = layers[lyr].fDimension;
    var filterValue;
    if (layers[lyr].filter) {
      //This might be single or multiple.  Its an array.
      filterValue = layers[lyr].filter;
    }

    var bubbleMeasureObj;
    if (layers[lyr].layer == 'bubble') {
      // console.log(layers[lyr]);
      bubbleMeasureObj = self.getMeasureAsArray(dsRows, dsMeasure, layers[lyr]);

      for (var key in bubbleMeasureObj) {
        bubbleMeasureObj[key] = util.unique(bubbleMeasureObj[key]);
      }
    }

    try {
      //Iterating two objects with index.
      //var keys = Object.keys(dsRowsKey);
      var text = Object.keys(dsRows);
      var len = text.length;
      var count = 0;
      //var prop;
      //It will be incremented only if sample mode is selected.
      var previewCount = 1;
      //Iterating the complete datasource to plot markeres in map.
      var dataArr = null;

      // iterate layer data
      while (count < len) {
        var lat, lng;
        //validating No filter is applied / filter is applied
        if (filter == 'No Filter' || $.inArray(dsRows[text[count]][filter], filterValue) > -1) {
          lat = null;
          lng = null;
          var address = null;
          var dataArr = null;
          //Changing data based on latlng / address
          if (component.addOrLatLong() == 'latlng' || (component.isDataProvider() && dataConfigs[dataProviderID].addOrLatLong == 'latlng')) {
            lat = dsRows[text[count]][latitudeIndex];
            lng = dsRows[text[count]][longitudeIndex];
          } else if (component.addOrLatLong() == 'address' || (component.isDataProvider() && dataConfigs[dataProviderID].addOrLatLong == 'address')) {
            var isContinue = self.plotAddressGeoCoding(self, component, dsRowsKey, dsRows, lat, lng, address, dataArr, text, count, addressIndexes);
            if (typeof isContinue == "boolean") {
              continue;
            }
            lat = isContinue.lat;
            lng = isContinue.lng;
          }
          //If Latitude and Longitude is not a number then incrementing the loop and continuing it.
          if (isNaN(lat) || isNaN(lng)) {
            count++;
            continue;
          }

          if (layers[lyr].layer == 'cluster' || layers[lyr].layer == 'marker') {

            //var pushed = false;
            var isConditionMatched = false;
            var isCondition = false; //Code Quality
            //Below if is Completely for conditional formatting - If and only if the conditional formatting is undefined it will work.
            if (layers[lyr].cf) {
              //Added the below for condition which helps to iterate multiple condition format for single layer.
              var cValue, cfColor = null,
                cfSymbol, cfSymCol, layerName, layerType, currentCond;
              for (var cond in layers[lyr].cf) {
                var evalCond = self.conditionalFormattingColor(cValue, cfColor, cfSymbol, cfSymCol, isCondition, isConditionMatched, layers, dsRows, cond, lyr, text, count);
                cValue = evalCond.cValue;
                cfColor = evalCond.cfColor;
                cfSymCol = evalCond.cfSymCol;
                cfSymbol = evalCond.cfSymbol;
                layerName = evalCond.layerName;
                layerType = evalCond.layerType;
                currentCond = evalCond.currentCond;
                isCondition = evalCond.isCondition;
                //If Icon type is only standard I am changing the markerIcon object.
                if (isCondition && currentCond.ruleType != 'dim' && layers[lyr].name == layerName && (layers[lyr].iconType == 'standard' || layers[lyr].iconType == 'font')) {
                  markerIcon = null;
                  if (layers[lyr].iconType == 'standard') {
                    markerIcon = L.SCNDesignStudioMarkers.icon({
                      markerColor: cfColor || 'blue',
                      selectedColor: layers[lyr].selectedColor,
                      iconSize: [layers[lyr].iconHeight || 32, layers[lyr].iconWidth || 32],
                      anchorPosition: [0.5, 0.5]
                    });
                  } else if (layers[lyr].iconType == 'font') {
                    markerIcon = L.FontAwesomeMarker.icon({
                      markerColor: cfColor || 'blue',
                      selectedColor: layers[lyr].selectedColor,
                      iconSize: [layers[lyr].iconHeight || 32, layers[lyr].iconWidth || 32],
                      anchorPosition: [0.5, 0.5],
                      iconClass: layers[lyr].iconClass,
                      iconColor: layers[lyr].iconColor
                    });
                  }
                  //console.log(1);
                } else if (isCondition && currentCond.ruleType != 'dim' && layers[lyr].name == layerName && layers[lyr].iconType == 'symbol') {
                  markerIcon = null;
                  if (currentCond.ruleType == 'targetVal') {
                    markerIcon = L.AwesomeMarkers.icon({
                      icon: cfSymbol,
                      markerColor: cfSymCol,
                      prefix: 'fa'
                    });
                  } else {
                    markerIcon = L.AwesomeMarkers.icon({
                      icon: currentCond.markerSymbol,
                      markerColor: currentCond.markerColor,
                      prefix: 'fa'
                    });
                  }
                  //console.log(2);
                } else if (isCondition && (layers[lyr].iconType == 'standard' || layers[lyr].iconType == 'font') && currentCond.ruleType == 'dim') {
                  markerIcon = null;
                  //console.log(cfColor);
                  if (layers[lyr].iconType == 'standard') {
                    markerIcon = L.SCNDesignStudioMarkers.icon({
                      markerColor: cfColor || 'blue',
                      selectedColor: layers[lyr].selectedColor,
                      iconSize: [layers[lyr].iconHeight || 32, layers[lyr].iconWidth || 32],
                      anchorPosition: [0.5, 0.5]
                    });
                  } else if (layers[lyr].iconType == 'font') {
                    markerIcon = L.FontAwesomeMarker.icon({
                      markerColor: cfColor || 'blue',
                      selectedColor: layers[lyr].selectedColor,
                      iconSize: [layers[lyr].iconHeight || 32, layers[lyr].iconWidth || 32],
                      anchorPosition: [0.5, 0.5],
                      iconClass: layers[lyr].iconClass,
                      iconColor: layers[lyr].iconColor
                    });
                  }
                } else if (isCondition && currentCond.ruleType != 'dim' && (layers[lyr].iconType == 'standard' || layers[lyr].iconType == 'font')) {
                  markerIcon = null;
                  if (layers[lyr].iconType == 'standard') {
                    markerIcon = L.SCNDesignStudioMarkers.icon({
                      markerColor: layers[lyr].color || 'blue',
                      selectedColor: layers[lyr].selectedColor,
                      iconSize: [layers[lyr].iconHeight || 32, layers[lyr].iconWidth || 32],
                      anchorPosition: [0.5, 0.5]
                    });
                  } else if (layers[lyr].iconType == 'font') {
                    markerIcon = L.FontAwesomeMarker.icon({
                      markerColor: layers[lyr].color || 'blue',
                      selectedColor: layers[lyr].selectedColor,
                      iconSize: [layers[lyr].iconHeight || 32, layers[lyr].iconWidth || 32],
                      anchorPosition: [0.5, 0.5],
                      iconClass: layers[lyr].iconClass,
                      iconColor: layers[lyr].iconColor
                    });
                  }

                } else if (isCondition && currentCond.ruleType == 'dim' && layers[lyr].iconType == 'symbol' && layers[lyr].name == layerName) {
                  markerIcon = null;
                  markerIcon = L.AwesomeMarkers.icon({
                    icon: currentCond.markerSymbol,
                    markerColor: currentCond.markerColor,
                    prefix: 'fa'
                  });
                }
              }
            }
            //while conditional formatting not applied
            if (!isCondition && !isConditionMatched) {

              if (layers[lyr].iconType == 'symbol') {
                markerIcon = null;
                markerIcon = L.AwesomeMarkers.icon({
                  icon: layers[lyr].symbol,
                  selectedColor: layers[lyr].symbolColorSelected,
                  markerColor: layers[lyr].symbolColor,
                  prefix: 'fa'
                });
                //console.log('Inside If');
              }
              if ((layers[lyr].iconType == 'standard' || layers[lyr].iconType == 'font')) {
                markerIcon = null;
                if (layers[lyr].iconType == 'standard') {
                  markerIcon = L.SCNDesignStudioMarkers.icon({
                    markerColor: layers[lyr].color || 'blue',
                    selectedColor: layers[lyr].selectedColor,
                    iconSize: [layers[lyr].iconHeight || 32, layers[lyr].iconWidth || 32],
                    anchorPosition: [0.5, 0.5]
                  });
                } else if (layers[lyr].iconType == 'font') {
                  markerIcon = L.FontAwesomeMarker.icon({
                    markerColor: layers[lyr].color || 'blue',
                    selectedColor: layers[lyr].selectedColor,
                    iconSize: [layers[lyr].iconHeight || 32, layers[lyr].iconWidth || 32],
                    anchorPosition: [0.5, 0.5],
                    iconClass: layers[lyr].iconClass,
                    iconColor: layers[lyr].iconColor
                  });
                }
              }
            }

            if (component.browser) {
              soloMarker = L.marker(new L.LatLng(parseFloat(lat), parseFloat(lng)), {
                icon: markerIcon,
                'type': layers[lyr].iconType,
                'index': count,
                'self': component,
                'locDS': datasource,
                'cEvent': false,
                'data': currentResultSet,
                'lat': latitudeIndex,
                'lng': longitudeIndex,
                'ds': layers[lyr].dataSource,
                'md5': md5(lat + lng) // jshint ignore:line
              });
            } else {
              soloMarker = L.marker(new L.LatLng(parseFloat(lat), parseFloat(lng)), {
                icon: markerIcon,
                'md5': md5(lat + lng) // jshint ignore:line
              });
            }
            //Adding No. Of Marker Tooltip
            noOfMarkers = noOfMarkers + 1;
            //$("#" + layers[lyr].name).attr("title", noOfMarkers + " Markers");
            //Marker Onclick the following scripting function will be called.
            if (component.browser) {
              soloMarker.on('click', function(mk) {

                var changingColor;

                if (mk.target.options.icon && !mk.target.options.icon.options.oldColor && component.isApplySelectedColor()) {
                  if (mk.target.options.type == 'standard') {
                    changingColor = L.SCNDesignStudioMarkers.icon({
                      markerColor: mk.target.options.icon.options.selectedColor || 'blue',
                      oldColor: mk.target.options.icon.options.markerColor,
                      iconSize: mk.target.options.icon.options.iconSize,
                      anchorPosition: [0.5, 0.5]
                    });
                    mk.target.setIcon(changingColor);
                  } else if (mk.target.options.type == 'font' && component.isApplySelectedColor()) {
                    changingColor = L.FontAwesomeMarker.icon({
                      markerColor: mk.target.options.icon.options.selectedColor || 'blue',
                      oldColor: mk.target.options.icon.options.markerColor,
                      iconSize: mk.target.options.icon.options.iconSize,
                      anchorPosition: [0.5, 0.5],
                      iconClass: mk.target.options.icon.options.iconClass,
                      iconColor: mk.target.options.icon.options.iconColor
                    });
                    mk.target.setIcon(changingColor);
                  } else if (mk.target.options.type == 'symbol' && component.isApplySelectedColor()) {
                    changingColor = L.AwesomeMarkers.icon({
                      icon: mk.target.options.icon.options.icon,
                      markerColor: mk.target.options.icon.options.selectedColor,
                      oldColor: mk.target.options.icon.options.markerColor,
                      prefix: 'fa'
                    });
                    mk.target.setIcon(changingColor);
                  }
                } else if (mk.target.options.icon && mk.target.options.icon.options.oldColor) {
                  var md5Lat = mk.target.options.md5;
                  var isLasso = false;
                  if (component.lassoSelected()) {
                    var lassoSel = JSON.parse(component.lassoSelected());
                    for (var l in lassoSel) {
                      var selMd5 = md5(lassoSel[l].lat + lassoSel[l].lng); // jshint ignore:line
                      if (selMd5 == md5Lat) {
                        isLasso = true;
                        break;
                      }
                    }
                  }
                  if (isLasso) {
                    return;
                  }
                  if (mk.target.options.type == 'standard' && component.isApplySelectedColor()) {
                    changingColor = L.SCNDesignStudioMarkers.icon({
                      markerColor: mk.target.options.icon.options.oldColor,
                      selectedColor: mk.target.options.icon.options.markerColor,
                      iconSize: mk.target.options.icon.options.iconSize,
                      anchorPosition: [0.5, 0.5]
                    });
                    mk.target.setIcon(changingColor);
                  } else if (mk.target.options.type == 'font' && component.isApplySelectedColor()) {
                    changingColor = L.FontAwesomeMarker.icon({
                      markerColor: mk.target.options.icon.options.oldColor,
                      selectedColor: mk.target.options.icon.options.markerColor,
                      iconSize: mk.target.options.icon.options.iconSize,
                      anchorPosition: [0.5, 0.5],
                      iconClass: mk.target.options.icon.options.iconClass,
                      iconColor: mk.target.options.icon.options.iconColor
                    });
                    mk.target.setIcon(changingColor);
                  } else if (mk.target.options.type == 'symbol' && component.isApplySelectedColor()) {
                    changingColor = L.AwesomeMarkers.icon({
                      icon: mk.target.options.icon.options.icon,
                      markerColor: mk.target.options.icon.options.oldColor,
                      selectedColor: mk.target.options.icon.options.markerColor,
                      prefix: 'fa'
                    });
                    mk.target.setIcon(changingColor);
                  }
                }
                setTimeout(function() {
                  if (!component.isSpiderfied) {
                    scripting.callDSScripting(mk);
                  }
                }, 200);
              });
            }
            if (layers[lyr].layer == 'marker' && component.isSpiderifyEnabled() && component.browser) {
              //Adding the markers for spiderfy effect.
              oms.addMarker(soloMarker);
            }
            var dataLabel = layers[lyr].dataLabel;
            var numberFormat = layers[lyr].nf;
            if (!numberFormat) {
              numberFormat = component.numberFormattingOptions;
            }
            if (component.enableDataLabel() && dataLabel !== 'Choose Measure' && dsRows[text[count]][dataLabel] !== undefined) {
              var dataLabelIndex = dataLabel;
              var measureVal = parseFloat(dsRows[text[count]][dataLabelIndex]);
              var scalingNumber = 1;
              for(var i=0; i<scalingFactor[dataLabel];i++){
                scalingNumber *= 10;
              }
              if (numberFormat.locDisplayScalingFactor) {
                if (numberFormat.locScalingFactor === 'default' && scalingFactor[dataLabel] && scalingFactor[dataLabel] !== 0) {
                  measureVal = measureVal;
                } else if (numberFormat.locScalingFactor != 'default') {
                  measureVal = (measureVal*scalingNumber) / parseFloat(numberFormat.locScalingFactor);
                }
              }
              if (numberFormat.mantissa > 0) {
                measureVal = measureVal.toFixed(numberFormat.mantissa);
              }
              if (numberFormat.locDecimalSeparator && measureVal.toString().indexOf('.') > 0) {
                var lastIndex = measureVal.lastIndexOf('.');
                var str = measureVal.split("");
                str[lastIndex] = numberFormat.locDecimalSeparator;
                measureVal = str.join("");
              }
              if (numberFormat.locThousandSeparator) {
                var decimalpoint = '.';
                if (numberFormat.locDecimalSeparator && numberFormat.locDecimalSeparator !== '.') {
                  decimalpoint = numberFormat.locDecimalSeparator;
                }
                measureVal = self.getUtil().putThousandsSeparators(measureVal, numberFormat.locThousandSeparator, decimalpoint);
              }
              if (numberFormat.locDisplayUnit && measureUnit[dataLabel]) {
                if (numberFormat.locUnitPlacement === 'right') {
                  measureVal = measureVal + ' ' + measureUnit[dataLabel];
                } else if (numberFormat.locUnitPlacement === 'left') {
                  measureVal = measureUnit[dataLabel] + ' ' + measureVal;
                } else {
                  measureVal = measureUnit[dataLabel] + '          ' + measureVal;
                }
              }

              if (numberFormat.locDisplayScalingFactor) {
                if (numberFormat.locScalingFactor === 'default' && scalingFactor[dataLabel] && scalingFactor[dataLabel] !== 0) {
                  if(scalingNumber>1){
                    measureVal = measureVal +" (* "+ scalingNumber+")";
                  }
                } else if (numberFormat.locScalingFactor != 'default') {
                  measureVal = measureVal +" (* "+ parseFloat(numberFormat.locScalingFactor)+")";
                }
              }

              var dataLabelText = numberFormat.locDatalabelPrefix + ' ' + measureVal + ' ' + numberFormat.locDatalabelSuffix;
              //Makes tooltip open on mouse over.
              soloMarker.bindTooltip(dataLabelText, {
                permanent: true
              });
            }

            //Popup text
            if (component.browser && toolTipIndex !== 'Dimension or Measure' && dsRows[text[count]][toolTipIndex] !== undefined) {
              var toolTipIndexes = toolTipIndex;

              if (dsMeasure[toolTipIndexes]) {
                toolTipIndexes = toolTipIndexes + 'format';
              }
              tootlTipText = dsTextName[toolTipIndex] + ' : ' + self.getUtil().parseTextValue(dsRows[text[count]][toolTipIndexes], dsRowsKey[text[count]][toolTipIndexes]);
              soloMarker.bindPopup(tootlTipText);
              //On Mouse hover the popup will open
              soloMarker.on('mouseover', function() {
                this.openPopup();
              });
              //On mouse out the popup will close
              soloMarker.on('mouseout', function() {
                this.closePopup(dsRows[text[count]]);
              });
            }
            //L.marker([lat, lng]).addTo(self.getMapContainer());
            //Adding solomarker to its group.
            soloMarker.addTo(self.groupOfMarker);
            if (layers[lyr].layer == 'marker') {
              soloMarker.addTo(self.clusterGroupMarker);
            }
            component.soloMarkerArr[md5(lat + lng + layers[lyr].name)] = soloMarker; // jshint ignore:line
            //Pushing all LatLng to array for autocentering feature
            component.arrayOfLatLng.push([
              lat,
              lng
            ]);
            if (component.browser) {
              //Adding it to the usedLayer array which helps in Lasso
              component.usedPoints[layers[lyr].name + count] = {
                'ttip': toolTipIndex,
                'dim': dsDimension,
                'msr': dsMeasure,
                'match': dsTextName,
                'text': dsRows[text[count]],
                'key': dsRowsKey[text[count]],
                'lat': lat,
                'lng': lng,
                'name': layers[lyr].name
              };
            }
          } //cluster / marker if ends
          else if (layers[lyr].layer == 'heat') {
            markerIcon.addLatLng(new L.LatLng(lat, lng));
            markerIcon.addTo(self.groupOfMarker);
            //Pushing all LatLng to array for autocentering feature
            component.arrayOfLatLng.push([
              lat,
              lng
            ]);
            //Adding it to the usedLayer array which helps in Lasso
            component.usedPoints[layers[lyr].name + count] = {
              'ttip': toolTipIndex,
              'dim': dsDimension,
              'msr': dsMeasure,
              'match': dsTextName,
              'text': dsRows[text[count]],
              'key': dsRowsKey[text[count]],
              'lat': lat,
              'lng': lng,
              'name': layers[lyr].name
            };
          } else if (layers[lyr].layer == 'bubble') {
            var color = '#4d86e2';
            //This if supports legacy
            if (layers[lyr].colorForSize && layers[lyr].colorForSize && layers[lyr].colorForSize != 'No Fill') {
              color = layers[lyr].colorForSize;
            }
            if (layers[lyr].bubbleColorForSize && layers[lyr].bubbleColorForSize === 'rgba(255, 255, 255, 0)' || layers[lyr].bubbleColorForSize === '') {
              color = 'transparent';
            } else if (layers[lyr].bubbleColorForSize) {
              color = layers[lyr].bubbleColorForSize;
            }
            var size = 15;
            var opacity = 1;
            if (layers[lyr].colorMeasure != 'Choose Measure') {
              switch (layers[lyr].colorConfig) {
                case 'dynamic':
                  var dynamicColor = layers[lyr].color;
                  var measureValue = dsRows[text[count]][layers[lyr].colorMeasure];
                  var minValue = Math.min.apply(Math, bubbleMeasureObj[layers[lyr].colorMeasure]);
                  var maxValue = Math.max.apply(Math, bubbleMeasureObj[layers[lyr].colorMeasure]);
                  var measurePercentage = ((measureValue - minValue) / (maxValue - minValue)) * 100;

                  for (var key in dynamicColor) {
                    if (measurePercentage >= dynamicColor[key].start && measurePercentage <= dynamicColor[key].stop) {
                      color = dynamicColor[key].color;
                      opacity = dynamicColor[key].opacity;
                      break;
                    }
                  }

                  break;
                case 'static':
                  var userColor = layers[lyr].color;
                  var clrMeasure = dsRows[text[count]][layers[lyr].colorMeasure];
                  for (var col in userColor) {
                    if (clrMeasure >= userColor[col].start && clrMeasure <= userColor[col].stop) {
                      color = userColor[col].color;
                      opacity = userColor[col].opacity;
                      break;
                    }
                  }

                  break;
                case 'brewer':
                  var brewerColor = layers[lyr].brewerColor;
                  var colorScale;
                  if (layers[lyr].natureOfData != 'stan') {
                    colorScale = chroma.scale(brewerColor).colors(_.parseInt(layers[lyr].buckets));
                  } else if (layers[lyr].natureOfData == 'stan') {
                    colorScale = chroma.scale(component.standardColor[brewerColor]).colors(_.parseInt(layers[lyr].buckets));
                  }

                  var scale = d3.scale.quantile().domain(bubbleMeasureObj[layers[lyr].colorMeasure]).range(colorScale);
                  color = scale(dsRows[text[count]][layers[lyr].colorMeasure]);
                  break;
              }
            }

            if (layers[lyr].sizeMeasure != 'Choose Measure') {
              var measureValue = dsRows[text[count]][layers[lyr].sizeMeasure];
              switch (layers[lyr].sizeConfig.type) {
                case 'pixel':

                  var minValue = Math.min.apply(Math, bubbleMeasureObj[layers[lyr].sizeMeasure]);
                  var maxValue = Math.max.apply(Math, bubbleMeasureObj[layers[lyr].sizeMeasure]);
                  var minPixel = parseFloat(layers[lyr].sizeConfig.min);
                  var maxPixel = parseFloat(layers[lyr].sizeConfig.max);
                  var diffPixel = maxPixel - minPixel;
                  var diffVal = maxValue - minValue;
                  var diffPer = diffPixel / diffVal;
                  size = ((measureValue - minValue) * diffPer) + minPixel;
                  break;
                case 'percentage':
                  var fullScreenValue = Math.abs(component.$().height() * component.$().width());
                  var minValue = Math.min.apply(Math, bubbleMeasureObj[layers[lyr].sizeMeasure]);
                  var maxValue = Math.max.apply(Math, bubbleMeasureObj[layers[lyr].sizeMeasure]);
                  var minPercentArea = layers[lyr].sizeConfig.min * (fullScreenValue / 100);
                  var maxPercentArea = layers[lyr].sizeConfig.max * (fullScreenValue / 100);
                  //0.32 - piR^2 - For finding the area.
                  var minPixel = Math.sqrt(minPercentArea * 0.32);
                  var maxPixel = Math.sqrt(maxPercentArea * 0.32);
                  var diffPixel = maxPixel - minPixel;
                  var diffVal = maxValue - minValue;
                  var diffPer = diffPixel / diffVal;
                  size = (((measureValue - minValue) * diffPer) + minPixel) / 10;

                  break;
              }
            }

            color = self.bubbleLayerConditionalFormatting(layers, dsRows, color, lyr, text, count);

            if (layers[lyr].selectedColor === undefined) {
              layers[lyr].selectedColor = color;
            } else if (layers[lyr].selectedColor === 'rgba(255, 255, 255, 0)' || layers[lyr].selectedColor === '') {
              layers[lyr].selectedColor = 'transparent';
            }

            if (size > 0) {
              noOfMarkers = noOfMarkers + 1;
              bubbleMarker = new L.RegularPolygonMarker(new L.LatLng(lat, lng), {
                color: 'black',
                opacity: 0.2,
                weight: 1,
                fillColor: color,
                fillOpacity: parseFloat(opacity),
                numberOfSides: 100,
                rotation: 0,
                selectedColor: layers[lyr].selectedColor,
                type: 'bubble',
                radius: parseFloat(size),
                'index': count,
                'self': component,
                'locDS': datasource,
                'cEvent': false,
                'data': currentResultSet,
                'lat': latitudeIndex,
                'lng': longitudeIndex,
                'ds': layers[lyr].dataSource,
                'md5': md5(lat + lng) // jshint ignore:line
              });

              component.soloMarkerArr[md5(lat + lng + layers[lyr].name)] = bubbleMarker; // jshint ignore:line

              bubbleMarker.on('click', function(bb) {
                if (bb.target.options && !bb.target.options.oldColor) {
                  bb.target.options.oldColor = bb.target.options.fillColor;
                  bb.target.setStyle({
                    'fillColor': bb.target.options.selectedColor
                  });
                } else if (bb.target.options && bb.target.options.oldColor) {
                  bb.target.setStyle({
                    'fillColor': bb.target.options.oldColor
                  });
                  delete bb.target.options.oldColor;
                }
                scripting.callDSScripting(bb);
              });

              //var dataLabel = layers[lyr].dataLabel;
              if (component.browser && component.enableDataLabel() && component.dataLabelIndex() !== 'Choose Measure' && dsRows[text[count]][component.dataLabelIndex()] !== undefined) {
                var dataLabelIndex = component.dataLabelIndex();
                var measureVal = parseFloat(dsRows[text[count]][dataLabelIndex]);

                if (component.locDisplayScalingFactor()) {
                  if (component.locScalingFactor() === 'default' && scalingFactor[component.dataLabelIndex()] && scalingFactor[component.dataLabelIndex()] !== 0) {
                    measureVal = measureVal / parseFloat(scalingFactor[component.dataLabelIndex()]);
                  } else if (component.locScalingFactor() != 'default') {
                    measureVal = measureVal / parseFloat(component.locScalingFactor());
                  }
                }
                if (component.mantissa() > 0) {
                  measureVal = measureVal.toFixed(component.mantissa());
                }
                if (component.locDecimalSeparator() && measureVal.toString().indexOf('.') > 0) {
                  var lastIndex = measureVal.lastIndexOf('.');
                  var str = measureVal.split("");
                  str[lastIndex] = component.locDecimalSeparator();
                  measureVal = str.join("");
                }
                if (component.locThousandSeparator()) {
                  var decimalpoint = '.';
                  if (component.locDecimalSeparator() && component.locDecimalSeparator() !== '.') {
                    decimalpoint = component.locDecimalSeparator();
                  }
                  measureVal = self.getUtil().putThousandsSeparators(measureVal, component.locThousandSeparator(), decimalpoint);
                }
                if (component.locDisplayUnit()) {
                  if (component.locUnitPlacement() === 'right') {
                    measureVal = measureVal + ' ' + measureUnit[component.dataLabelIndex()];
                  } else if (component.locUnitPlacement() === 'left') {
                    measureVal = measureUnit[component.dataLabelIndex()] + ' ' + measureVal;
                  } else {
                    measureVal = measureUnit[component.dataLabelIndex()] + '          ' + measureVal;
                  }
                }

                var dataLabelText = component.locDatalabelPrefix() + ' ' + measureVal + ' ' + component.locDatalabelSuffix();
                //Makes tooltip open on mouse over.
                bubbleMarker.bindTooltip(dataLabelText, {
                  permanent: true
                });
              }
              var popupContent = '';
              if (dsTextName[layers[lyr].sizeMeasure]) {
                var toolTipIndex = layers[lyr].sizeMeasure;
                if (dsMeasure[layers[lyr].sizeMeasure]) {
                  toolTipIndex = layers[lyr].sizeMeasure + 'format';
                }
                popupContent = popupContent + '<b>' + dsTextName[layers[lyr].sizeMeasure] + '<b> : <i>' + dsRows[text[count]][toolTipIndex] + '</i></br>';
              }

              if (dsTextName[layers[lyr].colorMeasure]) {
                var toolTipIndex = layers[lyr].colorMeasure;
                if (dsMeasure[layers[lyr].colorMeasure]) {
                  toolTipIndex = layers[lyr].colorMeasure + 'format';
                }
                popupContent = popupContent + '<b>' + dsTextName[layers[lyr].colorMeasure] + '<b> : <i>' + dsRows[text[count]][toolTipIndex] + '</i>';
              }
              //Makes tooltip open on mouse over.
              bubbleMarker.bindPopup(popupContent);
              toolTipIndex = toolTipIndex.replace('format', '');
              //On Mouse hover the popup will open
              bubbleMarker.on('mouseover', function() {
                this.openPopup();
              });
              //On mouse out the popup will close
              bubbleMarker.on('mouseout', function() {
                this.closePopup(dsRows[text[count]]);
              });

              bubbleMarker.addTo(self.groupOfMarker);
              //Pushing all LatLng to array for autocentering feature
              component.arrayOfLatLng.push([
                lat,
                lng
              ]);
              //Adding it to the usedLayer array which helps in Lasso
              component.usedPoints[layers[lyr].name + count] = {
                'ttip': toolTipIndex,
                'dim': dsDimension,
                'msr': dsMeasure,
                'match': dsTextName,
                'text': dsRows[text[count]],
                'key': dsRowsKey[text[count]],
                'lat': lat,
                'lng': lng,
                'name': layers[lyr].name
              };
            }
          }
          //Push to the aggregate array
          if (layers[lyr].layer != 'heat') {
            if ($.inArray(toolTipIndex, Object.keys(measures)) > -1) {
              var filterKey = parseFloat(lat) + '_' + parseFloat(lng);
              aggregateArray.push({
                value: parseFloat(dsRows[text[count]][toolTipIndex]),
                key: filterKey
              });
            }
          }
          //If Sample mode activated.
          if (component.samplePreview()) {
            if (previewCount == sampleCount) {
              break;
            }

            previewCount = previewCount + 1;
          }
        }
        //filter if ends
        lat = null;
        lng = null;
        count++;
      } //iterating datasource while loop ends

    } catch (e) {
      //Possible scenarios are
      // 1. If lat/lng is not valid
      // 2. If Resultset is displayed
      if (window.console) {
        console.log(e); // jshint ignore:line
      }
    }
    //if(layers[lyr].layer!="bubble"){

    if (layers[lyr].isCluster && layers[lyr].layer == "marker" && noOfMarkers > layers[lyr].clusterLimit) {
      self.groupOfMarker = self.clusterGroupMarker;
    }
    if (component.markerReference[layers[lyr].layer + '|' + layers[lyr].name] == undefined) {
      component.layerControl.addOverlay(self.groupOfMarker, layers[lyr].name);
      //If Heat layer we are bringing it's z-index property below
      $('.leaflet-heatmap-layer').css('z-index', '-10');
      //Adds layer to the map.
    self.getMapContainer().addLayer(self.groupOfMarker);
      //self.getMapContainer().addLayer(self.groupOfMarker);
      component.markerReference[layers[lyr].layer + '|' + layers[lyr].name] = self.groupOfMarker;

      //For Migration
      if (undefined === layers[lyr].isLoad) {
        layers[lyr].isLoad = true;
      }

      if (window.localStorage.layerVisibility !== undefined && !JSON.parse(window.localStorage.layerVisibility)[layers[lyr].name]) {
        layers[lyr].isLoad = false;
      }

      if (!layers[lyr].isLoad) {
        $(this).removeClass('selected');
        self.getMapContainer().removeLayer(self.groupOfMarker);
        component.layerVisibility[layers[lyr].name] = false;
      }
      //Adding all Layers name to an array
      if ($.inArray(layers[lyr].name, component.allLayers) == -1) {
        component.allLayers.push(layers[lyr].name);
      }
      //}
      if (layers[lyr].layer == 'bubble') {}
    }
  }
  //backing up for reset
  component.backUpLayerVisibility = JSON.stringify(component.layerVisibility);
  //iterating different layers for loop ends
  //Adding Stringified data to leafletLayers object and commiting it..
  component.leafletLayers(component.allLayers);

  //Aggregation based on selection.
  component.aggregateResults = vbiCharts.aggregate(component.ttipAggregation(), aggregateArray, 'key');
  return layers;
};

MarkerLayer.prototype.drawMarker = function() {
  var self = this;
  var component = self.getProps();
  //If user didn't enable multiple dataprovider then making array empty
  if (!component.isDataProvider()) {
    component.dataProviderArray = [];
    component.dataProviderArray.push('VBX_DEFAULT_DS');
  }

  component.isFunctionRan = true;
  self.createLegend(component);

  //Based on User's action it will hide layer control icon.
  if (component.isHideLayerControl()) {
    $('.leaflet-control-layers').css('visibility', 'hidden');
  } else {
    $('.leaflet-control-layers').css('visibility', 'visible');
  }

  //iterating all the multiple data's
  for (var z = 0; z < component.dataProviderArray.length; z++) {
    //If Z is 1 then user is using multiple datasource.  so clearing all the backup's
    if (z > 0) {
      component.backUpMarkerLayer = null;
      component.backUpClusterLayer = null;
      component.backUpHeatLayer = null;
      component.backUpBubbleLayer = null;
    }
    var isContinueIteration = self.iteratePerDataProvider(self, component, z);
    if (isContinueIteration == undefined) {
      break;
    }
  }
  if (component.browser) {
    component.isLayerByScripting(false);
    component.firePropertiesChanged(['isLayerByScripting']);
  }

};





MarkerLayer.prototype.unique = function(origArr) {
  var newArr = [],
    origLen = origArr.length,
    found, x, y;
  for (x = 0; x < origLen; x++) {
    found = undefined;
    for (y = 0; y < newArr.length; y++) {
      if (origArr[x] === newArr[y]) {
        found = true;
        break;
      }
    }

    if (!found) {
      newArr.push(origArr[x]);
    }
  }

  return newArr;
};

MarkerLayer.prototype.getMeasureAsArray = function(data, measure, currentLayer) {
  var measureArray = {};
  for (var key in measure) {
    measureArray[key] = [];
  }

  for (var key in data) {
    if (currentLayer && currentLayer.fDimension != 'No Filter' && currentLayer.filter && currentLayer.filter.length > 0) {
      for (var fil in currentLayer.filter) {
        if (currentLayer.filter[fil] == data[key][currentLayer.fDimension]) {
          for (var keys in measure) {
            measureArray[keys].push(data[key][keys]);
          }
        }
      }
    } else {
      for (var keys in measure) {
        measureArray[keys].push(data[key][keys]);
      }
    }
  }

  return measureArray;
};

MarkerLayer.prototype.addBubbleToMap = function(data, layer, measureKV) {
  var self = this;
  var component = self.getProps();
  var primaryMeasure = measureKV[layer.sizeMeasure];
  var secondaryMeasure = measureKV[layer.colorMeasure];
  var dataLayerOptions = {
    recordsField: null,
    latitudeField: component.latitudeIndex(),
    longitudeField: component.longitudeIndex(),
    displayOptions: {},
    layerOptions: {
      numberOfSides: 50,
      color: '#000000',
      fillOpacity: 0.7,
      opacity: 1,
      weight: 0.4,
      gradient: true,
      dropShadow: false
    },
    tooltipOptions: {
      iconSize: new L.Point(100, 80),
      iconAnchor: new L.Point(-5, 100)
    }
  };
  dataLayerOptions.displayOptions[primaryMeasure] = {
    radius: new L.LinearFunction(new L.Point(0, 5), new L.Point(100, 20))
  };
  dataLayerOptions.displayOptions[secondaryMeasure] = {
    fillColor: new L.HSLHueFunction(new L.Point(50, 60), new L.Point(100, 0))
  };
  var dataLayer = new L.DataLayer(data, dataLayerOptions);
  try {
    component.layerControl.addOverlay(dataLayer, layer.name);
    self.getMapContainer().addLayer(dataLayer);
    component.markerReference[layer.name] = dataLayer;
    if (!layer.isLoad) {
      $(this).removeClass('selected');
      self.getMapContainer().removeLayer(dataLayer);
      component.layerVisibility[layer.layer + '|' + layer.name] = false;
    }
    //Adding all Layers name to an array
    if ($.inArray(layer.name, component.allLayers) == -1) {
      component.allLayers.push(layer.name);
    }
  } catch (e) {
    if (window.console) {
      console.log(e); // jshint ignore:line
    }
  }
};
/**
 * Method will get key/text as argument where in it will take country, state/region, city, zipcode and combines everyone and it returns
 *
 * @param {key, text} The key/text is a object which has keys and texts repectively.
 * @return {combinedAddress} .
 */
MarkerLayer.prototype.getAddress = function(key, text, addressIndexes) {
  var self = this;

  var country = addressIndexes.countryIndex != 'Dimension or Measure' ? addressIndexes.countrySwitch == 'key' ? key[addressIndexes.countryIndex] : self.getUtil().parseTextValue(text[addressIndexes.countryIndex], key[addressIndexes.countryIndex]) : '';
  var stateRegion = addressIndexes.stateIndex != 'Dimension or Measure' ? addressIndexes.stateSwitch == 'key' ? key[addressIndexes.stateIndex] : self.getUtil().parseTextValue(text[addressIndexes.stateIndex], key[addressIndexes.stateIndex]) : '';
  var city = addressIndexes.cityIndex != 'Dimension or Measure' ? addressIndexes.citySwitch == 'key' ? key[addressIndexes.cityIndex] : self.getUtil().parseTextValue(text[addressIndexes.cityIndex], key[addressIndexes.cityIndex]) : '';
  var postalCode = addressIndexes.zipIndex != 'Dimension or Measure' ? addressIndexes.zipSwitch == 'key' ? key[addressIndexes.zipIndex] : self.getUtil().parseTextValue(text[addressIndexes.zipIndex], key[addressIndexes.zipIndex]) : '';
  var combinedAddress = city + ',' + stateRegion + ',' + country + ',' + postalCode;
  return combinedAddress;
};
/**
 * The method will create marker object based on its type with various user options.
 *
 * @param {layer} Layer object might have marker/cluster/heat.
 * @return {marker} .
 */
MarkerLayer.prototype.getMarker = function(layer) {
  var self = this;
  var component = self.getProps();

  var marker = null;
  if (layer.layer == 'heat') {
    var gradient = {};
    if (layer.isColorScheme) {
      if (layer.colorChoice == 'manual') {
        var colorArray = [];
        for (var c in layer.quantile) {
          colorArray.push(layer.quantile[c]);
        }
        if (colorArray.length == 1) {
          colorArray.push(colorArray[0]);
        }
        var colorScale = chroma.scale(colorArray).colors(_.parseInt(layer.buckets));
        var gr = 1 / _.parseInt(layer.buckets);
        for (var g = 0; g < colorScale.length; g++) {
          gradient[gr.toFixed(1)] = colorScale[g];
          gr = gr + (1 / _.parseInt(layer.buckets));
        }
      } else {
        var colorScale = chroma.scale(layer.brewerColor).colors(_.parseInt(layer.buckets));
        var gr = 1 / _.parseInt(layer.buckets);
        for (var g = 0; g < colorScale.length; g++) {
          gradient[gr.toFixed(1)] = colorScale[g];
          gr = gr + (1 / _.parseInt(layer.buckets));
        }
      }
    } else {
      gradient = layer.gradient;
    }

    marker = L.heatLayer([], {
      minOpacity: _.parseInt(layer.minOpacity),
      maxZoom: _.parseInt(layer.maxZoom),
      max: _.parseInt(layer.max),
      radius: _.parseInt(layer.radius),
      blur: _.parseInt(layer.blur),
      gradient: gradient
    });
    self.setGroupOfMarker(new L.layerGroup());
  } else if (layer.layer == 'cluster') {
    //A static marker if cluster layer is selected.
    marker = L.SCNDesignStudioMarkers.icon({
      markerColor: 'blue',
      iconSize: [
        32,
        32
      ],
      anchorPosition: [
        0.5,
        0.5
      ]
    });
    var markerCluster = L.markerClusterGroup({
      spiderfyOnMaxZoom: layer.spiderfyOnMaxZoom,
      showCoverageOnHover: layer.showCoverageOnHover,
      zoomToBoundsOnClick: layer.zoomToBoundsOnClick,
      removeOutsideVisibleBounds: layer.removeOutsideVisibleBounds,
      disableClusteringAtZoom: layer.disableClusteringAtZoom,
      maxClusterRadius: layer.maxClusterRadius,
      spiderfyDistanceMultiplier: layer.spiderfyDistanceMultiplier
    });
    self.setGroupOfMarker(markerCluster);
  } else if (layer.layer == 'marker') {
    var coverage = layer.clusterCoverage || true;
    var radius = layer.clusterRadius || 80;
    var markerCluster = L.markerClusterGroup({
      showCoverageOnHover: coverage,
      maxClusterRadius: radius
    });
    self.setClusterGroupOfMarker(markerCluster);
    if (layer.iconType == 'standard') {

      marker = L.SCNDesignStudioMarkers.icon({
        markerColor: layer.color || 'blue',
        selectedColor: layer.selectedColor,
        iconSize: [
          layer.iconHeight || 32,
          layer.iconWidth || 32
        ],
        anchorPosition: [
          0.5,
          0.5
        ]
      });
    } else if (layer.iconType == 'symbol') {
      marker = L.AwesomeMarkers.icon({
        selectedColor: layer.symbolColorSelected,
        icon: layer.symbol,
        markerColor: layer.symbolColor,
        prefix: 'fa'
      });
    } else if (layer.iconType == 'url') {
      var urlSize = layer.urlSize;
      marker = L.icon({
        iconUrl: layer.url,
        iconAnchor: [(urlSize.height || 32) / 2, (urlSize.width || 32)],
        anchorPosition: [0.5, 0.5],
        popupAnchor: [1, -32]
      });
    } else if (layer.iconType == 'font') {
      layer.color = layer.color == 'No Fill' ? 'transparent' : layer.color;
      layer.selectedColor = layer.selectedColor == 'No Fill' ? 'transparent' : layer.selectedColor;
      layer.iconColor = layer.iconColor == 'No Fill' ? 'transparent' : layer.iconColor;
      marker = L.FontAwesomeMarker.icon({
        markerColor: layer.color || 'blue',
        selectedColor: layer.selectedColor,
        iconSize: [
          layer.iconHeight || 32,
          layer.iconWidth || 32
        ],
        iconClass: layer.iconClass,
        iconColor: layer.iconColor
      });
    }
    if (!component.isEnableCollision()) {
      self.setGroupOfMarker(new L.layerGroup.collision({
        margin: _.parseInt(component.collisionDistance())
      }));
    } else {
      self.setGroupOfMarker(new L.layerGroup());
    }

  } else if (layer.layer == 'bubble') {
    if (!component.isEnableCollision()) {
      self.setGroupOfMarker(new L.layerGroup.collision({
        margin: _.parseInt(component.collisionDistance())
      }));
    } else {
      self.setGroupOfMarker(new L.layerGroup());
    }
  }

  return marker;
};
/**
 * Different map events are registered here.
 *
 * @return {null} .
 */
MarkerLayer.prototype.triggerEvents = function() {
  var self = this;
  var component = self.getProps();
  //self.getMapContainer().off('zoomstart');
  self.getMapContainer().on('zoomstart', function() {
    component.zoomLevel = self.getMapContainer().getZoom();
    component.mapZoomLevel(component.zoomLevel);
  });

  //self.getMapContainer().off('zoomend');
  self.getMapContainer().on('zoomend', function() {
    component.zoomLevel = self.getMapContainer().getZoom();
    component.mapZoomLevel(component.zoomLevel);
    var mapCenter = self.getMapContainer().getCenter();
    component.setInitialLatitude(mapCenter.lat);
    component.setInitialLongitude(mapCenter.lng);
  });

  self.getMapContainer().on('dragend', function() {
    var mapCenter = self.getMapContainer().getCenter();
    component.setInitialLatitude(mapCenter.lat);
    component.setInitialLongitude(mapCenter.lng);
  });

  self.getMapContainer().off('baselayerchange');
  self.getMapContainer().on('baselayerchange', function(e) {
    //console.log(Object.keys(component.baseLayers).indexOf(e.name));
    component.mapLayers(Object.keys(component.baseLayers).indexOf(e.name));
    //component.firePropertiesChanged(["mapLayers"]);
    //DSXR-1316 - commented for fixing the above issue.
    //Expecting issue because of the commented above line.
  });


  if (typeof component.layerRunTime() == 'boolean' && component.layerRunTime()) {
    if (component.baseLayerControl) {
      self.getMapContainer().removeControl(component.baseLayerControl);
      //self.getMapContainer().removeControl(component.googleBaseLayerControl);
    }
    component.baseLayerControl = L.control.layers(component.baseLayers).addTo(self.getMapContainer());
    //component.googleBaseLayerControl = L.control.layers(component.googleBaseLayers).addTo(self.getMapContainer());
  } else {
    if (component.baseLayerControl) {
      self.getMapContainer().removeControl(component.baseLayerControl);
      //self.getMapContainer().removeControl(component.googleBaseLayerControl);
      component.baseLayerControl = null;
      //component.googleBaseLayerControl = null;
    }
  }

  self.getMapContainer().off('overlayadd');
  self.getMapContainer().on('overlayadd', function(e) {
    if ($('#' + e.name + '-cho').length !== 0) {
      $('#' + e.name + '-cho').show();
    }

    $('.lName#' + e.name).css('background-color', 'transparent');
    $('.lName#' + e.name).css('opacity', '1');

    component.layerVisibility[e.name] = true;
    component.leafletLayerStatus(JSON.stringify(component.layerVisibility));
    if (component.browser) {
      component.firePropertiesChanged(['leafletLayerStatus']);
    }
    component.isLayerAdded = true;
    self.updateLassoObject();
  });

  self.getMapContainer().off('overlayremove');
  self.getMapContainer().on('overlayremove', function(e) {
    //This click will trigger unspiderfy event of overlappingmarker spiderfier.
    $('#' + e.target._container.id).click();
    if ($('#' + e.name + '-cho').length !== 0) {
      $('#' + e.name + '-cho').hide();
    }

    $('.lName#' + e.name).css('background-color', 'lightgray');
    $('.lName#' + e.name).css('opacity', '0.4');

    component.layerVisibility[e.name] = false;
    component.leafletLayerStatus(JSON.stringify(component.layerVisibility));
    component.isLayerAdded = true;
    if (component.browser) {
      component.firePropertiesChanged(['leafletLayerStatus']);
    }
    self.updateLassoObject();
  });
};

/**
 * Resetting the visibility of layers based on the object.
 *
 * @return {null}
 */
MarkerLayer.prototype.resetOverlay = function(layerVisibility) {
  var self = this;
  var component = self.getProps();
  var layerControl = {};
  //Separating and organizing the layers
  for (var lyr in component.layerControl._layers) {
    layerControl[component.layerControl._layers[lyr].name] = component.layerControl._layers[lyr].layer;
  }

  //Resetting the visibility of layers based on the object.
  for (var key in layerVisibility) {
    if (layerVisibility[key]) {
      $(this).addClass('selected');
      self.getMapContainer().addLayer(layerControl[key]);
      layerVisibility[key] = true;
    } else {
      $(this).removeClass('selected');
      self.getMapContainer().removeLayer(layerControl[key]);
      layerVisibility[key] = false;
    }
  }
};

/**
 * Based on hide/show of the layer the lasso objects will be updated.
 *
 * @return {null}
 */
MarkerLayer.prototype.updateLassoObject = function() {
  var self = this;
  var component = self.getProps();
  //Adding the objects in the objects based on the hide / show features
  //If lassoBounds has some value then lasso selection been made.
  //lasso selected markers
  var lassoSelected = [];
  if (Object.keys(component.lassoBound).length > 0) {
    var dsRows = component.lassoUpdatedUsedPoints;
    for (var key in component.lassoBound) {
      //Refer lasso.js "self.getMapContainer().on('draw:created', function(e) {" for more details
      var selectedBounds = component.lassoBound[key].selectedBounds;
      var bounds = component.lassoBound[key].bounds;
      //The latitude and longitude pints inside the minimal bounds are in this minimalDataset array.
      var minimalDataset = [];
      //dsRows is a array which contains all the points used in the map. Here it will help us the configure the minimal dataset in the said bound.
      for (var i in dsRows) {
        if (bounds.contains(new L.LatLng(dsRows[i].lat, dsRows[i].lng))) {
          minimalDataset.push(dsRows[i]);
        }
      }

      for (var i in minimalDataset) {
        var currentPoint = new L.LatLng(minimalDataset[i].lat, minimalDataset[i].lng);
        var oddNodes = false;
        var k, j = selectedBounds.length - 1;
        for (k = 0; k < selectedBounds.length; k++) {
          //fetch adjucent points of the polygon
          var polyK = selectedBounds[k];
          var polyJ = selectedBounds[j];
          //check the intersections
          if (polyK.lng > currentPoint.lng != polyJ.lng > currentPoint.lng && currentPoint.lat < (polyJ.lat - polyK.lat) * (currentPoint.lng - polyK.lng) / (polyJ.lng - polyK.lng) + polyK.lat) {
            oddNodes = !oddNodes;
          }
          //switch between odd and even
          j = k;
        }

        if (oddNodes) {
          //mouse point is inside the polygon
          if (component.layerVisibility[minimalDataset[i].name]) {
            lassoSelected.push(minimalDataset[i]);
          }
        } else {}
      } //console.log(JSON.stringify(lassoSelected));
    }

    component.lassoSelected(JSON.stringify(lassoSelected));
    component.isLassoScriptingDone = true;
    component.firePropertiesChanged(['lassoSelected']);
  }
};
/**
 * All the layers will be removed.
 *
 * @return {null}
 */
MarkerLayer.prototype.removeAllLayers = function() {
  var self = this;
  var component = self.getProps();
  //Removes the layer control icon from map and add's again
  //self.getMapContainer().removeControl(component.layerControl);
  //component.layerControl = L.control.layers();
  //component.layerControl.addTo(self.getMapContainer());
  //Iterating the existing layers and removing all from the map object
  if (Object.keys(component.markerReference).length > 0) {
    for (var lyr in component.markerReference) {
      var id = L.Util.stamp(component.markerReference[lyr]);
      var obj = self.getMapContainer()._layers[id];
      if (obj && self.getMapContainer().hasLayer(obj)) {
        self.getMapContainer().removeLayer(obj);
        delete self.getMapContainer()._layers[id];
      }
    }
  }

};
/**
 * Based on the users selection in scripting functions, the layers will be hidden/shown.  The layers name will be in Comma(,) separated.
 *
 * @return {null}.
 */
MarkerLayer.prototype.hideShowLayers = function() {
  var self = this;
  var component = self.getProps();
  var allLayers = component.allLayers;
  var layerControl = {};
  //Separating and organizing the layers
  for (var lyr in component.layerControl._layers) {
    layerControl[component.layerControl._layers[lyr].name] = component.layerControl._layers[lyr].layer;
  }
  //Uncomment for development purposes without going for scripting.
  /*component.isHideShowLayer(true);
    component.lyrName('A,b');
    component.isHideLayer('showOnly');*/
  //component.lyrName is the input given by the user as argument in the scripting part.  That value initialized in ztl.
  var hsLayers = JSON.parse(component.lyrName());
  for (var l in hsLayers) {
    var name = l.trim().split(',');
    var layersToWork = [];
    if (component.isHideShowLayer() || hsLayers[l] == 'hideOnly' || hsLayers[l] == 'showOnly') {
      for (var lyr in name) {
        if ($.inArray(name[lyr].trim(), allLayers) == -1) {
          alert('No layer has name : ' + name[lyr]); // jshint ignore:line
        } else {
          layersToWork.push(name[lyr].trim());
        }
      }
      //Resetting the visibility of layers based on the object.
      for (var key in component.layerVisibility) {
        if (component.layerVisibility[key]) {
          $(this).addClass('selected');
          self.getMapContainer().addLayer(layerControl[key]);
          component.layerVisibility[key] = true;
        } else {
          $(this).removeClass('selected');
          self.getMapContainer().removeLayer(layerControl[key]);
          component.layerVisibility[key] = false;
        }
      }

      switch (hsLayers[l]) {
        case 'hideOnly':
          for (var i = 0; i < layersToWork.length; i++) {
            if ($('#' + layersToWork[i] + '-cho').length !== 0) {
              $('#' + layersToWork[i] + '-cho').hide();
            }

            $(this).removeClass('selected');
            self.getMapContainer().removeLayer(layerControl[layersToWork[i]]);
            component.layerVisibility[layersToWork[i]] = false;
            var layerN = {};
            if (window.localStorage.layerVisibility === undefined) {
              layerN[layersToWork[i]] = false;
              window.localStorage.layerVisibility = JSON.stringify(layerN);
            } else {
              layerN = JSON.parse(window.localStorage.layerVisibility);
              layerN[layersToWork[i]] = false;
              window.localStorage.layerVisibility = JSON.stringify(layerN);
            }
          }

          break;
        case 'hideAll':
          for (var i = 0; i < allLayers.length; i++) {
            $(this).removeClass('selected');
            self.getMapContainer().removeLayer(layerControl[allLayers[i]]);
            component.layerVisibility[layersToWork[i]] = false;
            var layerN = {};
            if (window.localStorage.layerVisibility === undefined) {
              layerN[layersToWork[i]] = false;
              window.localStorage.layerVisibility = JSON.stringify(layerN);
            } else {
              layerN = JSON.parse(window.localStorage.layerVisibility);
              layerN[layersToWork[i]] = false;
              window.localStorage.layerVisibility = JSON.stringify(layerN);
            }
          }

          break;
        case 'showOnly':
          for (var i = 0; i < layersToWork.length; i++) {
            if ($('#' + layersToWork[i] + '-cho').length !== 0) {
              $('#' + layersToWork[i] + '-cho').show();
            }

            $(this).addClass('selected');
            self.getMapContainer().addLayer(layerControl[layersToWork[i]]);
            component.layerVisibility[layersToWork[i]] = true;
            var layerN = {};
            if (window.localStorage.layerVisibility === undefined) {
              layerN[layersToWork[i]] = true;
              window.localStorage.layerVisibility = JSON.stringify(layerN);
            } else {
              layerN = JSON.parse(window.localStorage.layerVisibility);
              layerN[layersToWork[i]] = true;
              window.localStorage.layerVisibility = JSON.stringify(layerN);
            }
          }

          break;
        case 'showAll':
          for (var i = 0; i < allLayers.length; i++) {
            $(this).addClass('selected');
            self.getMapContainer().addLayer(layerControl[allLayers[i]]);
            component.layerVisibility[allLayers[i]] = true;
            var layerN = {};
            if (window.localStorage.layerVisibility === undefined) {
              layerN[layersToWork[i]] = true;
              window.localStorage.layerVisibility = JSON.stringify(layerN);
            } else {
              layerN = JSON.parse(window.localStorage.layerVisibility);
              layerN[layersToWork[i]] = true;
              window.localStorage.layerVisibility = JSON.stringify(layerN);
            }
          }

          break;
      }
    }


  }
  if (component.isHideShowLayer()) {
    component.lyrName('');
    component.isHideShowLayer(false);
    component.isHideShow = true;
    component.firePropertiesChanged([
      'isHideShowLayer',
      'mapZoomLevel',
      'lyrName'
    ]);
  }
};
