/**
 * Function Name of each component specific series!
 * @global
 */
var _globalCidSeriesRef = {
  'columnBar': [
    {'familyBar': []},
    {'columnBarSeries': []}
  ],
  'radar': [
    {'familyBar': []},
    {'radarSeries': []}
  ],
  'polar': [
    {'familyBar': []},
    {'polarSeries': []}
  ],
  'area': [
    {'familyBar': []},
    {'areaSeries': []}
  ],
  'line': [
    {'familyBar': []},
    {'lineSeries': []}
  ],
  'pie': [
    {'familyPie': []},
    {'pieSeries': []}
  ],
  'donut': [
    {'familyPie': []},
    {'donutSeries': []}
  ],
  'semiCircleDonut': [
    {'familyPie': []},
    {'semiCircleDonutSeries': []}
  ],
  'funnel': [
    {'familyPie': []},
    {'funnelSeries': []}
  ],
  'dualAxes': [
    {'familyMultiAxes': []},
    {'dualAxesSeries': []}
  ],
  'multipleAxes': [
    {'familyMultiAxes': []},
    {'multipleAxesSeries': []}
  ],
  'bubble': [{'bubbleSeries': []}],
  'scatter': [{'scatterSeries': []}],
  'stackedArea': [
    {'familyBar': []},
    {'stackedAreaSeries': []}
  ],
  'stackedColumnBar': [
    {'familyBar': []},
    {'stackedColumnBarSeries': []}
  ],
  'combination': [{'combinationSeries': []}],
  'superCombination': [{'superCombinationSeries': []}],
  'fixedColumn': [{'fixedColumnSeries': []}],
  'heatMapChart': [{'heatMapChartSeries': []}],
  'groupStacked': [{'groupStackedSeries': []}],
  'waterfall': [
   
     {'piefamilyDrilldown': []},

    //{'familyDrilldown': []},
  {'waterfallSeries': []}

  ],
  'sparkline': [{'sparklineSeries': []}],
  'sparklineTable': [{'sparklineTableSeries': []}],
  'columnBarDD': [
    {'familyDrilldown': []},
    {'columnBarDrilldownSeries': []}
  ],
  'pieDD': [
    {'piefamilyDrilldown': []},
    {'pieDrilldownSeries': []}
  ],
  'funnelDD': [
    {'piefamilyDrilldown': []},
    {'funnelDrilldownSeries': []}
  ],
  'bullet': [{'bulletSeries': []}],
  'marimekko': [{'marimekkoSeries': []}],
  'boxPlotChart': [{'boxPlotChartSeries': []}],
  'treeMap': [{'treeMapSeries': []}],
  'treeMapDrilldown': [{'treeMapDrilldownSeries': []}]
};
/**
 * Function Name of each component specific dynamicUI!
 * @global
 */
var _globalCidDynamicUIRef = {
  'columnBar': [],
  'area': [],
  'radar': [],
  'polar': [],
  'line': [],
  'pie': [],
  'donut': [],
  'semiCircleDonut': [],
  'funnel': [],
  'dualAxes': [],
  'multipleAxes': [],
  'bubble': [],
  'scatter': [],
  'stackedArea': [],
  'stackedColumnBar': [],
  'combination': [],
  'superCombination': [],
  'supercontainer': [],
  'fixedColumn': [],
  'heatMapChart': [],
  'groupStacked': [],
  'waterfall': [],
  'sparkline': [],
  'sparklineTable': [],
  'columnBarDD': [],
  'pieDD': [],
  'funnelDD': [],
  'bullet': [],
  'marimekko': [],
  'treeMap': [],
  'treeMapDrilldown': []
};
/**
 * This method is used to attach specific series function to passed
 * argument. Argument passed should be the seriesLibrary object!
 * @param {object} that
 * @param {Boolean} isDynamicUI
 * @return {Boolean}
 * @global
 * @author Nilesh Bhandari
 */
function linkProperties(that, isDynamicUI) {
  // that refers to seriesLibrary object
  try {
    var cid = that.vbiCharts.component.CID;
    if (cid === undefined || cid == '') {
      throw {message: '<component reference>.CID value is unavailable'};
    }

    var argObj = '';
    if (isDynamicUI) {
      argObj = _globalCidDynamicUIRef[cid];
    } else {
      argObj = _globalCidSeriesRef[cid];
    }

    if (argObj !== undefined) {
      implementMethods(that, argObj);
      return true;
    } else {
      throw {message: 'Unable to extend lib for ' + cid + '\nReason : [' + cid + '] Object not found'};
    }
  } catch (e) {
    // alert('Error @ extend_series.linkProperties: '+ e.message);
    return false;
  }
}
/**
 * Attach additional properties & methods to the first argument as its
 * ownProperty. This function takes care of all error in the process of including property.
 * @param {Object} that - Object that gets the given properties & methods
 * @param {Object} argObj - Object of array containing implementing function name reference and list of function names to be attached
 */
function implementMethods(that, argObj) {
  // that refers to seriesLibrary object
  for (var i = 0; i < argObj.length; ++i) {
    for (key in argObj[i]) {
      try {
        var tempRef = window[key];
        if (typeof tempRef === 'function') {
          tempRef.call(that, argObj[i][key]);
        } else {
          throw {message: 'Skipped implementMethods for [' + key + ']\nReason: Function not found'};
        }
      } catch (e) {
        alert('Error @ extend_series.implementMethod: ' + e.message);
      }
    }
  }
}
/**
 * Search for a key in array
 * @param {Array} args
 * @param {String} searchKey
 * @return {Boolean} <code>true</code> - if found else <code>false</code>
 */
function validateImplementMethod(args, searchKey) {
  if (args.length == 0) {
    return true;
  }

  if (jQuery.inArray(searchKey, args) !== -1) {
    return true;
  }

  return false;
}
